@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2002 2003 2004 2005 2006 2007, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006 2007 2008 2009 2011, Magnus Hjorth
+ * Copyright (C) 2012, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -84,6 +85,7 @@ static void chunk_init(Chunk *object)
      object->format.type = DATAFORMAT_PCM;
      object->format.samplerate=44100;
      object->format.samplesize=2;
+     object->format.packing=0;
      object->format.channels=2;
      object->format.sign=1;
      object->format.bigendian = IS_BIGENDIAN;
@@ -94,9 +96,9 @@ static void chunk_init(Chunk *object)
      chunks = g_list_append( chunks, object );
 }
 
-guint chunk_get_type(void)
+GtkType chunk_get_type(void)
 {
-     static guint id=0;
+     static GtkType id=0;
      if (!id) {
 	  GtkTypeInfo info = {
 	       "Chunk",
@@ -258,31 +260,43 @@ Chunk *chunk_convert_sampletype(Chunk *chunk, Dataformat *newtype)
      return r;
 }
 
-static gint channel_to_remove, current_channel;
-
-static gboolean chunk_remove_channel_proc(void *in, guint samplesize, 
-					  chunk_writeout_func out_func, 
-					  WriteoutID id,
-					  Dataformat *format, 
-					  Dataformat *outformat)
+static Chunk *chunk_ds_remap(Chunk *chunk, int new_channels, int *map)
 {
-     guint c = current_channel;
-     current_channel++;
-     if (current_channel == format->channels) current_channel = 0;
-     if (c != channel_to_remove) return out_func(id,in,samplesize);
-     else return FALSE;
+     GList *l;
+     DataPart *dp;
+     Datasource *new_ds;
+     Chunk *new_chunk,*new_chunk_part;
+     Chunk *res = NULL, *c;
+     for (l=chunk->parts; l!=NULL; l=l->next) {
+	  dp = (DataPart *)l->data;
+	  new_ds = datasource_channel_map(dp->ds,new_channels,map);
+	  new_chunk = chunk_new_from_datasource(new_ds);
+	  new_chunk_part = chunk_get_part(new_chunk,dp->position,dp->length);
+	  gtk_object_sink(GTK_OBJECT(new_chunk));
+	  if (res == NULL) {
+	       res = new_chunk_part;
+	  } else {
+	       c = chunk_append(res,new_chunk_part);
+	       gtk_object_sink(GTK_OBJECT(res));
+	       gtk_object_sink(GTK_OBJECT(new_chunk_part));
+	       res = c;
+	  }
+     }
+     g_free(map);
+     return res;
 }
 
 Chunk *chunk_remove_channel(Chunk *chunk, gint channel, StatusBar *bar)
 {
-     Dataformat fmt;
-     channel_to_remove = channel;
-     current_channel = 0;
-     memcpy(&fmt,&(chunk->format),sizeof(Dataformat));
-     fmt.channels --;
-     return chunk_filter_tofmt(chunk,chunk_remove_channel_proc,NULL,
-			       CHUNK_FILTER_ONE,FALSE,&fmt,DITHER_NONE,bar,
-			       _("Removing channel"));
+     int *map,i,new_chans;
+
+     new_chans = chunk->format.channels - 1;
+     map = g_malloc(new_chans * sizeof(int));
+     for (i=0; i<new_chans; i++) {
+	  if (i >= channel) map[i]=i+1;
+	  else map[i]=i;
+     }
+     return chunk_ds_remap(chunk, new_chans, map);
 }
 
 gboolean chunk_dump(Chunk *chunk, EFILE *file, gboolean bigendian, 
@@ -343,9 +357,8 @@ static gboolean convert_back_write(WriteoutID id, gpointer data, guint length)
      struct convert_back *cbp = (struct convert_back *)id;
      guint i;
      i = length / sizeof(sample_t);
-     *(cbp->clipcount) += unnormalized_count(data,i);
      convert_array(data,&dataformat_sample_t,cbp->buf,cbp->tofmt,i,
-		   cbp->dither_mode);
+		   cbp->dither_mode, cbp->clipcount);
      return tempfile_write(cbp->tmp,cbp->buf,i*cbp->tofmt->samplesize);
 }
 
@@ -403,8 +416,8 @@ Chunk *chunk_filter_tofmt(Chunk *chunk, chunk_filter_tofmt_proc proc,
      convert_back = convert && !chunk_filter_use_floating_tempfiles;
 
      if (convert) {
-	  informat.type = outformat.type = DATAFORMAT_FLOAT;
-	  informat.samplesize = outformat.samplesize = sizeof(sample_t);
+	  memcpy(&informat,&dataformat_sample_t,sizeof(Dataformat));
+	  memcpy(&outformat,&dataformat_sample_t,sizeof(Dataformat));
 	  informat.channels = chunk->format.channels;
 	  outformat.channels = tofmt->channels;
 	  informat.samplebytes = informat.samplesize * informat.channels;
@@ -639,19 +652,16 @@ Chunk *chunk_onechannel(Chunk *chunk, int dither_mode, StatusBar *bar)
 Chunk *chunk_copy_channel(Chunk *chunk, gint channel, int dither_mode, 
 			  StatusBar *bar)
 {
-     Chunk *r;
-     gboolean *map;
-     int i;
+     int i,new_chans;
+     int *map;
 
-     map = g_malloc0(chunk->format.channels * (chunk->format.channels+1) * 
-		     sizeof(gboolean));
-     for (i=0; i<chunk->format.channels; i++)
-	  map[i*(chunk->format.channels+1) + i] = TRUE;
-     map[channel*(chunk->format.channels+1) + (chunk->format.channels)] = TRUE;
-     r = chunk_remap_channels(chunk,chunk->format.channels+1,map,dither_mode,
-			      bar);
-     g_free(map);
-     return r;
+     new_chans = chunk->format.channels + 1;
+     map = g_malloc(new_chans * sizeof(int));
+     for (i=0; i<new_chans; i++) {
+	  if (i <= channel) map[i] = i;
+	  else map[i] = i-1;
+     }
+     return chunk_ds_remap(chunk,new_chans,map);
 }
 
 
@@ -681,6 +691,7 @@ Chunk *chunk_mix(Chunk *c1, Chunk *c2, int dither_mode, StatusBar *bar)
      memcpy(&format,&(c1->format),sizeof(Dataformat));
      format.type = DATAFORMAT_FLOAT;
      format.samplesize = sizeof(sample_t);
+     format.bigendian = dataformat_sample_t.bigendian;
      format.samplebytes = format.samplesize * format.channels;
 
      /* Prepare for processing */
@@ -761,8 +772,226 @@ Chunk *chunk_mix(Chunk *c1, Chunk *c2, int dither_mode, StatusBar *bar)
      gtk_object_sink(GTK_OBJECT(c));
 
      return d;
+#undef BUFLEN
 }
 
+static void sandwich_copy(char *dest_buf, int dest_offset, int dest_item_size,
+			  char *source_buf, int source_item_size, int items)
+{
+     int i;
+     for (i=0; i<items; i++)
+	  memcpy(dest_buf+dest_offset+i*dest_item_size, 
+		 source_buf+i*source_item_size,
+		 source_item_size);
+}
+
+
+static Chunk *chunk_sandwich_main(Chunk *c1, Chunk *c2, int dither_mode, StatusBar *bar)
+{
+#define BUFLEN 512
+     off_t u,mixlen;
+     guint chn1,chn2,x,x1,x2;
+     off_t clipcount = 0;
+     char *buf1,*buf2,*outbuf;
+     Dataformat format;
+     Chunk *m;
+     ChunkHandle *ch1, *ch2;
+     TempFile tmp;
+
+     g_assert(dataformat_samples_equal(&(c1->format),&(c2->format)) && 
+	      c1->format.samplerate == c2->format.samplerate);
+
+     /* Number of channels */
+     chn1 = c1->format.channels;
+     chn2 = c2->format.channels;
+
+     /* Number of multi-channel samples to mix */
+     g_assert(c1->length == c2->length);
+     mixlen = c1->length;
+
+     memcpy(&format,&(c1->format),sizeof(Dataformat));
+     format.channels = chn1+chn2;
+     format.samplebytes = format.samplesize * format.channels;
+
+     /* Prepare for processing */
+     ch1 = chunk_open(c1);
+     if ( ch1 == NULL ) return NULL;
+     ch2 = chunk_open(c2);
+     if ( ch2 == NULL ) { chunk_close(ch1); return NULL; }
+     tmp = tempfile_init(&format,FALSE);
+     status_bar_begin_progress(bar, mixlen, _("Combining channels"));
+
+     buf1 = g_malloc(BUFLEN*c1->format.samplebytes);
+     buf2 = g_malloc(BUFLEN*c2->format.samplebytes);
+     outbuf = g_malloc(BUFLEN*format.samplebytes);
+
+     for (u=0; u<mixlen; u+=x) {
+
+	  x = BUFLEN;
+	  if ((off_t)x > (mixlen-u))
+	       x = ((guint)(mixlen-u));
+
+	  x1 = chunk_read_array(ch1,u,x*c1->format.samplebytes,buf1,
+				dither_mode,&clipcount);
+	  x2 = chunk_read_array(ch2,u,x*c2->format.samplebytes,
+				buf2,dither_mode,&clipcount);
+	  
+	  if (x1 == 0 || x2 == 0) {
+	       tempfile_abort(tmp);
+	       chunk_close(ch1);
+	       chunk_close(ch2);
+	       status_bar_end_progress(bar);
+	       g_free(buf1);
+	       g_free(buf2);
+	       g_free(outbuf);
+	       return NULL;
+	  }
+
+	  g_assert(x1 == x*c1->format.samplebytes &&
+		   x2 == x*c2->format.samplebytes);
+	  
+	  /* Combine the data from buf1,buf2 into outbuf */
+	  sandwich_copy(outbuf,0,format.samplebytes,
+			buf1,c1->format.samplebytes, x);
+	  sandwich_copy(outbuf,c1->format.samplebytes,format.samplebytes,
+			buf2,c2->format.samplebytes, x);
+
+	  /* Write outbuf */
+	  if (tempfile_write(tmp,outbuf,x*format.samplebytes)
+	      || status_bar_progress(bar,x)) {
+	       tempfile_abort(tmp);
+	       chunk_close(ch1);
+	       chunk_close(ch2);
+	       status_bar_end_progress(bar);
+	       g_free(buf1);
+	       g_free(buf2);
+	       g_free(outbuf);
+	       return NULL;
+	  }
+     }    
+
+     /* Finish processing */
+     chunk_close(ch1);
+     chunk_close(ch2);
+     m = tempfile_finished(tmp);
+     status_bar_end_progress(bar);
+     g_free(buf1);
+     g_free(buf2);
+     g_free(outbuf);
+     if (!m) return NULL;
+
+     /* Warn if clipping occured */
+     clipwarn(clipcount,FALSE);
+
+     return m;
+#undef BUFLEN
+}
+
+Chunk *chunk_sandwich(Chunk *c1, Chunk *c2, 
+		      off_t c1_align_point, off_t c2_align_point,
+		      off_t *align_point_out, int dither_mode, StatusBar *bar)
+{
+     Chunk *before, *mainpart_c1, *mainpart_c2, *mainpart_sw, *after;
+     Chunk *c,*d,*c1_remap,*c2_remap;
+     int *map1,*map2,outchans,i;
+     int mp_c1_maxlen,mp_c2_maxlen,mp_c1_offset,mp_c2_offset,mp_len;
+     off_t offset;
+
+     offset = c2_align_point-c1_align_point;     
+     outchans = c1->format.channels + c2->format.channels;
+
+     map1 = g_malloc(outchans * sizeof(map1[0]));
+     map2 = g_malloc(outchans * sizeof(map2[0]));
+     for (i=0; i<c1->format.channels; i++) { map1[i]=i; map2[i]=-1; }
+     for (; i<outchans; i++) { map1[i]=-1; map2[i]=i-(c1->format.channels); }
+
+     c1_remap = chunk_ds_remap(c1,outchans,map1);
+     c2_remap = chunk_ds_remap(c2,outchans,map2);
+
+     /* g_free(map1); g_free(map2); Done already by chunk_ds_remap */
+
+     if (offset == 0) {
+	  before = NULL;
+	  mp_c1_maxlen = c1->length;
+	  mp_c1_offset = 0;
+	  mp_c2_maxlen = c2->length;
+	  mp_c2_offset = 0;
+	  *align_point_out = c1_align_point;
+     } else if (offset > 0) {
+	  before = chunk_get_part(c2_remap, 0, offset);
+	  mp_c1_maxlen = c1->length;
+	  mp_c1_offset = 0;
+	  mp_c2_maxlen = c2->length - offset;
+	  mp_c2_offset = offset;
+	  *align_point_out = c2_align_point;
+     } else {
+	  before = chunk_get_part(c1_remap, 0, -offset);
+	  mp_c1_maxlen = c1->length+offset;
+	  mp_c1_offset = -offset;
+	  mp_c2_maxlen = c2->length;
+	  mp_c2_offset = 0;
+	  *align_point_out = c1_align_point;
+     }
+
+     mp_len = MIN(mp_c1_maxlen, mp_c2_maxlen);
+     if (mp_len > 0) {
+	  mainpart_c1 = chunk_get_part(c1, mp_c1_offset, mp_len);
+	  mainpart_c2 = chunk_get_part(c2, mp_c2_offset, mp_len);
+     } else 
+	  mainpart_c1 = mainpart_c2 = NULL;
+
+     if (mp_len < mp_c1_maxlen) {	  
+	  after = chunk_get_part(c1_remap, mp_c1_offset+mp_len, 
+				 mp_c1_maxlen-mp_len);
+     } else if (mp_len < mp_c2_maxlen) {
+	  after = chunk_get_part(c2_remap, mp_c2_offset+mp_len,
+				 mp_c2_maxlen-mp_len);
+     } else {
+	  after = NULL;
+     }
+
+     /* Floating refs: c1_remap,c2_remap,before,after,mainpart_c1,mainpart_c2 */
+     gtk_object_sink(GTK_OBJECT(c1_remap));
+     gtk_object_sink(GTK_OBJECT(c2_remap));
+
+     if (mp_len > 0) {
+	  mainpart_sw = chunk_sandwich_main(mainpart_c1, mainpart_c2, 
+					    dither_mode, bar);
+	  gtk_object_sink(GTK_OBJECT(mainpart_c1));
+	  gtk_object_sink(GTK_OBJECT(mainpart_c2));
+	  if (mainpart_sw == NULL) {
+	       if (before != NULL) gtk_object_sink(GTK_OBJECT(before));
+	       if (after != NULL) gtk_object_sink(GTK_OBJECT(after));
+	       return NULL;
+	  }
+     } else 
+	  mainpart_sw = NULL;
+     
+     /* Floating refs: before,after,mainpart_sw */
+
+     c = before;
+     if (mainpart_sw != NULL) {
+	  if (c != NULL) {
+	       d = chunk_append(c,mainpart_sw);
+	       gtk_object_sink(GTK_OBJECT(c));
+	       gtk_object_sink(GTK_OBJECT(mainpart_sw));
+	       c = d;
+	  } else
+	       c = mainpart_sw;	  
+     }
+     if (after != NULL) {
+	  if (c != NULL) {
+	       d = chunk_append(c,after);
+	       gtk_object_sink(GTK_OBJECT(c));
+	       gtk_object_sink(GTK_OBJECT(after));
+	       c = d;
+	  } else
+	       c = after;
+     }
+     return c;
+}
+
+
 static struct {
      sample_t *buf;
      int bufsize;
@@ -811,46 +1040,6 @@ static gboolean chunk_remap_channels_mixmode_proc(void *in,
      return out_func(id, chunk_remap_channels_data.buf, 
 		     samps*outformat->samplebytes);     
 }
-
-static gboolean chunk_remap_channels_rawmode_proc(void *in, 
-						  guint sample_size,
-						  chunk_writeout_func out_func,
-						  WriteoutID id,
-						  Dataformat *informat,
-						  Dataformat *outformat)
-{
-     int samps,ssize=informat->samplesize,i,j,k;
-     char *iptr, *optr;
-
-     /* Make sure we have room in buffer */
-     samps = sample_size / informat->samplebytes;
-     if (samps * outformat->samplebytes > chunk_remap_channels_data.bufsize) {
-	  g_free(chunk_remap_channels_data.buf);
-	  chunk_remap_channels_data.buf = 
-	       g_malloc(samps*outformat->samplebytes);
-	  chunk_remap_channels_data.bufsize = samps*outformat->samplebytes;
-     }
-
-     iptr = (char *)in;
-     optr = (char *)chunk_remap_channels_data.buf;
-
-     /* This could be optimized further (replace memcpy) */
-
-     for (i=0; i<samps; i++) {
-	  for (j=0; j<outformat->channels; j++) {
-	       k = chunk_remap_channels_data.map[j*(informat->channels+1)];
-	       if (k >= 0)
-		    memcpy(optr, iptr+k*ssize,ssize);
-	       else
-		    memset(optr, 0, ssize);
-	       optr += ssize;
-	  }
-	  iptr += informat->samplebytes;
-     }
-
-     return out_func(id, chunk_remap_channels_data.buf, 
-		     samps*outformat->samplebytes);
-}
 						  
 Chunk *chunk_remap_channels(Chunk *chunk, int channels_out, gboolean *map, 
 			    int dither_mode, StatusBar *bar)
@@ -860,6 +1049,7 @@ Chunk *chunk_remap_channels(Chunk *chunk, int channels_out, gboolean *map,
      int channels_in = chunk->format.channels;
      Dataformat tofmt;
      Chunk *r;
+     int *bigmap,*smallmap;
      
      memcpy(&tofmt,&(chunk->format),sizeof(Dataformat));
      tofmt.channels = channels_out;
@@ -868,32 +1058,36 @@ Chunk *chunk_remap_channels(Chunk *chunk, int channels_out, gboolean *map,
      chunk_remap_channels_data.buf = NULL;
      chunk_remap_channels_data.bufsize = 0;
 
-     /* Generate the map */
+     /* Generate the maps */
      mixmode = FALSE;
-     chunk_remap_channels_data.map = g_malloc((channels_in+1)*channels_out*
-					      sizeof(int));
+     bigmap = g_malloc((channels_in+1)*channels_out*sizeof(int));
+     smallmap = g_malloc(channels_out*sizeof(int));
+
      for (i=0; i<channels_out; i++) {
+	  smallmap[i] = -1;
 	  k = 0;
 	  for (j=0; j<channels_in; j++) 
-	       if (map[j*channels_out + i])
-		    chunk_remap_channels_data.map[i*(channels_in+1)+(k++)] = j;
-	  chunk_remap_channels_data.map[i*(channels_in+1) + k] = -1;
+	       if (map[j*channels_out + i]) {
+		    bigmap[i*(channels_in+1)+(k++)] = j;
+		    smallmap[i] = j;
+	       }
+	  bigmap[i*(channels_in+1) + k] = -1;
 	  if (k > 1) mixmode = TRUE;
      }
 
      if (mixmode) {
+	  chunk_remap_channels_data.map = bigmap;
+	  g_free(smallmap);
 	  r = chunk_filter_tofmt(chunk,chunk_remap_channels_mixmode_proc,
 				 NULL,CHUNK_FILTER_MANY,TRUE,&tofmt,
 				 dither_mode,bar,_("Mixing channels"));
+	  g_free(chunk_remap_channels_data.buf);
+	  g_free(chunk_remap_channels_data.map);
      } else {
-	  r = chunk_filter_tofmt(chunk,chunk_remap_channels_rawmode_proc,
-				 NULL,CHUNK_FILTER_MANY,FALSE,&tofmt,
-				 dither_mode,bar,_("Mapping channels"));
-	  
+	  g_free(bigmap);
+	  r = chunk_ds_remap(chunk,channels_out,smallmap);
      }
      
-     g_free(chunk_remap_channels_data.buf);
-     g_free(chunk_remap_channels_data.map);
      return r;
 }
 
@@ -992,13 +1186,19 @@ guint chunk_read_array(ChunkHandle *handle, off_t sampleno,
      GList *l;     
      DataPart *dp;
      guint r=0,i,j;
+     guint sb = handle->format.samplebytes;
      off_t o;
-     for (l=handle->parts;l!=NULL;l=l->next) {
+
+     /* Loop until we have collected enough data or reached EOF
+      * The condition size >= sb handles the case size is not an even
+      * multiple of the number of sample bytes (24-bit data usually) */
+     
+     for (l=handle->parts; l!=NULL && size>=sb; l=l->next) {
 	  dp = (DataPart *)l->data;
 	  if (dp->length > sampleno) {
 	       /* Calculate required size if it's larger than we need. */
 	       o = dp->length - sampleno;	       
-	       o *= (off_t)(handle->format.samplebytes);
+	       o *= (off_t)sb;
 	       if (o < (off_t)size) j = (guint)o;
 	       else j = size;
 	       /* Read data. */
@@ -1010,7 +1210,6 @@ guint chunk_read_array(ChunkHandle *handle, off_t sampleno,
 	       sampleno = 0;
 	       size -= i;
 	       buffer = ((gchar *)buffer) + i;
-	       if (size==0 || l->next==NULL) return r;
 	  } else
 	       sampleno -= dp->length;
      }
@@ -1595,55 +1794,18 @@ Chunk *chunk_byteswap(Chunk *chunk)
      return d;
 }
 
-static gboolean chunk_convert_channels_down_proc(void *in, guint sample_size,
-						 chunk_writeout_func out_func,
-						 WriteoutID id, 
-						 Dataformat *informat, 
-						 Dataformat *outformat)
+Chunk *chunk_convert_channels(Chunk *chunk, guint new_channels)
 {
-     return out_func(id,in,outformat->samplebytes);
-}
-
-static guint zero_level_buf_len;
-static gchar zero_level_buf[32];
-
-static gboolean chunk_convert_channels_up_proc(void *in, guint sample_size,
-					       chunk_writeout_func out_func,
-					       WriteoutID id,
-					       Dataformat *informat,
-					       Dataformat *outformat)
-{
-     if (zero_level_buf_len == 0) {
-	  sample_t s[8] = { 0 };
-	  zero_level_buf_len = outformat->samplebytes - informat->samplebytes;
-	  convert_array(s,&dataformat_sample_t,zero_level_buf,outformat,
-			outformat->channels-informat->channels, DITHER_NONE);
-	  /* printf("zero_level_buf_len = %d\n",zero_level_buf_len); */
-     }
-     return (out_func(id,in,informat->samplebytes) || 
-	     out_func(id,zero_level_buf,zero_level_buf_len));
-}
-
-Chunk *chunk_convert_channels(Chunk *chunk, guint new_channels, 
-			      int dither_mode, StatusBar *bar)
-{
-     Dataformat outformat;
+     int *map;
+     int i;
      g_assert(chunk->format.channels != new_channels);
-     memcpy(&outformat,&(chunk->format),sizeof(Dataformat));
-     outformat.channels = new_channels;
-     outformat.samplebytes = outformat.channels * outformat.samplesize;
-     if (new_channels < chunk->format.channels)
-	  return chunk_filter_tofmt(chunk,chunk_convert_channels_down_proc,
-				    NULL,CHUNK_FILTER_FULL,FALSE,&outformat,
-				    dither_mode, 
-				    bar,_("Removing channels"));
-     else {
-	  zero_level_buf_len = 0;
-	  return chunk_filter_tofmt(chunk,chunk_convert_channels_up_proc,
-				    NULL,CHUNK_FILTER_FULL,FALSE,&outformat,
-				    dither_mode, 
-				    bar,_("Adding channels"));
-     }
+     map = g_malloc(new_channels * sizeof(int));
+     for (i=0; i<new_channels; i++) map[i] = i;
+     /* When converting mono files, put mono channel in both left and right. 
+      * This is in most cases what we want. */
+     if (chunk->format.channels==1 && new_channels > 1)
+	  map[1] = 0;
+     return chunk_ds_remap(chunk,new_channels,map);
 }
 
 Chunk *chunk_convert(Chunk *chunk, Dataformat *new_format, 
@@ -1658,8 +1820,7 @@ Chunk *chunk_convert(Chunk *chunk, Dataformat *new_format,
 	       conv_driver = rateconv_driver_id(FALSE,0);
 	  if (new_format->channels < chunk->format.channels) {
 	       /* First decrease channels, then convert rate */
-	       d = chunk_convert_channels(chunk,new_format->channels,
-					  dither_mode,bar);
+	       d = chunk_convert_channels(chunk,new_format->channels);
 	       if (d == NULL) return NULL;
 	       c = chunk_convert_samplerate(d,new_format->samplerate,
 					    conv_driver,dither_mode,bar);
@@ -1671,8 +1832,7 @@ Chunk *chunk_convert(Chunk *chunk, Dataformat *new_format,
 	       d = chunk_convert_samplerate(chunk,new_format->samplerate,
 					    conv_driver,dither_mode,bar);
 	       if (d == NULL) return NULL;
-	       c = chunk_convert_channels(d,new_format->channels,dither_mode,
-					  bar);
+	       c = chunk_convert_channels(d,new_format->channels);
 	       gtk_object_sink(GTK_OBJECT(d));
 	       if (c == NULL) return NULL;
 
@@ -1684,8 +1844,7 @@ Chunk *chunk_convert(Chunk *chunk, Dataformat *new_format,
 	  }
      } else if (new_format->channels != chunk->format.channels) {
 	  /* Just convert channels */
-	  c = chunk_convert_channels(chunk,new_format->channels,
-				     dither_mode,bar);
+	  c = chunk_convert_channels(chunk,new_format->channels);
 	  if (c == NULL) return NULL;
      } 
 
