@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006 2008 2010 2012, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -36,6 +36,10 @@ typedef struct {
      guint32 samplerate;         /* Samples/sec */
      unsigned samplesize;           /* bytes (PCM: 1,2,3 or 4, 
 				  * float: sizeof(gfloat) or sizeof(gdouble))*/
+     int packing;               /* For non-packed 24-in-32 samples,
+				 * set samplesize to 4 and packing to 1 (data in
+				 * MSB) or 2 (data in LSB)
+				 * For other types, this should be 0 */
      int channels;              /* Number of channels (1-8) */
      int samplebytes;           /* should always be samplesize*channels */
 
@@ -48,7 +52,13 @@ typedef struct {
 extern Dataformat dataformat_sample_t;
 extern Dataformat dataformat_single;
 
-extern gboolean ieee_le_compatible;
+#define CONVERT_MODE_BIASED    0
+#define CONVERT_MODE_NOOFFS    1
+#define CONVERT_MODE_MAX       1
+
+extern gint sample_convert_mode;
+
+extern gboolean ieee_le_compatible,ieee_be_compatible;
 
 #define FORMAT_IS_SAMPLET(fmtp) ((fmtp)->type == DATAFORMAT_FLOAT && (fmtp)->samplesize == sizeof(sample_t))
 
@@ -69,6 +79,9 @@ void dataformat_save_to_inifile(gchar *ini_prefix, Dataformat *format,
 				gboolean full);
 
 
+const gchar *sampletype_name(Dataformat *fmt);
+
+
 /* -------------------
  * CONVERSION ROUTINES 
  * ------------------- */
@@ -88,7 +101,7 @@ typedef gfloat sample_t;
 #endif
 
 #define DITHER_NONE       0
-#define DITHER_TRIANGULAR 1
+#define DITHER_MINIMAL    1
 #define DITHER_MAX        1
 /* This constant can be used to indicate that dithering should not
  * be necessary. If this is given when an FP->PCM conversion is done,
@@ -102,12 +115,16 @@ typedef gfloat sample_t;
  */
 
 #define maximum_float_value(x) (1.0)
+sample_t minimum_float_value(Dataformat *x);
 
 void convert_array(void *indata, Dataformat *indata_format,
 		   void *outdata, Dataformat *outdata_format,
-		   guint count, int dither_mode);
+		   guint count, int dither_mode, off_t *clipcount);
 
-gint unnormalized_count(sample_t *buf, gint buflen);
+void apply_convert_factor(Dataformat *infmt, Dataformat *outfmt,
+			  sample_t *buffer, guint count);
+
+gint unnormalized_count(sample_t *buf, gint buflen, Dataformat *target);
 
 void conversion_selftest(void);
 void conversion_performance_test(void);
