@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006 2009 2010 2012, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -51,6 +51,7 @@ static void datasource_init(Datasource *obj)
      obj->format.type = DATAFORMAT_PCM;
      obj->format.samplerate=44100;
      obj->format.samplesize=2;
+     obj->format.packing=0;
      obj->format.channels=2;
      obj->format.sign=1;
      obj->format.bigendian = IS_BIGENDIAN;
@@ -58,12 +59,17 @@ static void datasource_init(Datasource *obj)
      obj->length = 0;
      obj->bytes = 0;
      obj->opencount = 0;
+     obj->read_temparea.ptr = NULL;
+     obj->read_temparea.size = 0;
+     obj->readfp_temparea.ptr = NULL;
+     obj->readfp_temparea.size = 0;
      obj->tag = 0;
 }
 
 static void datasource_clear(Datasource *ds)
 {
      g_assert(ds->opencount == 0);
+     g_assert(ds->read_temparea.ptr == NULL && ds->readfp_temparea.ptr == NULL);
 
      switch (ds->type) {
 
@@ -105,6 +111,16 @@ static void datasource_clear(Datasource *ds)
 	  if (ds->data.clone) 
 	       gtk_object_unref(GTK_OBJECT(ds->data.clone));
 	  ds->data.clone = NULL;
+	  break;
+
+     case DATASOURCE_CHANMAP:
+	  if (ds->data.chanmap.clone)
+	       gtk_object_unref(GTK_OBJECT(ds->data.chanmap.clone));
+	  if (ds->data.chanmap.map)
+	       g_free(ds->data.chanmap.map);
+	  ds->data.chanmap.clone = NULL;
+	  ds->data.chanmap.map = NULL;
+
      }
      
      ds->type = DATASOURCE_SILENCE;
@@ -141,6 +157,16 @@ GtkType datasource_get_type(void)
      return id;
 }
 
+static char *datasource_get_temparea(struct temparea *ta, int size)
+{
+     if (size > ta->size) {
+	  g_free(ta->ptr);
+	  ta->ptr = g_malloc(size);
+	  ta->size = size;
+     }
+     return ta->ptr;
+}
+
 /* Creates a copy of the original datasource
  * If it refers to a file, create a DATASOURCE_REF instead.  
  */
@@ -176,6 +202,15 @@ static Datasource *datasource_copy(Datasource *orig)
 	  ds->data.clone = orig->data.clone;
 	  gtk_object_ref(GTK_OBJECT(ds->data.clone));
 	  gtk_object_sink(GTK_OBJECT(ds->data.clone));
+	  break;
+     case DATASOURCE_CHANMAP:
+	  ds->data.chanmap.clone = orig->data.chanmap.clone;
+	  gtk_object_ref(GTK_OBJECT(ds->data.chanmap.clone));
+	  gtk_object_sink(GTK_OBJECT(ds->data.chanmap.clone));
+	  ds->data.chanmap.map = g_malloc(ds->format.channels * sizeof(int));
+	  memcpy(ds->data.chanmap.map, orig->data.chanmap.map, 
+		 ds->format.channels*sizeof(int));
+	  break;
      }
      return ds;
 }
@@ -184,8 +219,6 @@ Datasource *datasource_clone_df(Datasource *source, Dataformat *format)
 {
      Datasource *df;
 
-     g_assert(source->format.type == DATAFORMAT_PCM);
-
      if (source->type == DATASOURCE_REAL ||
 	 source->type == DATASOURCE_SILENCE ||
 	 source->type == DATASOURCE_CLONE) {
@@ -222,6 +255,29 @@ Datasource *datasource_byteswap(Datasource *source)
      return ds;
 }
 
+Datasource *datasource_channel_map(Datasource *source, int n_channels,
+				   int *map)
+{
+     Datasource *ds;
+
+     if (source == NULL) return NULL;
+
+     ds = gtk_type_new(datasource_get_type());
+     ds->type = DATASOURCE_CHANMAP;
+     memcpy(&(ds->format),&(source->format),sizeof(Dataformat));
+     ds->format.channels = n_channels;
+     ds->format.samplebytes = ds->format.samplesize * n_channels;
+     ds->length = source->length;
+     ds->bytes = ds->length * ds->format.samplebytes;
+     ds->data.chanmap.clone = source;
+     gtk_object_ref(GTK_OBJECT(source));
+     gtk_object_sink(GTK_OBJECT(source));
+     ds->data.chanmap.map = g_malloc(n_channels * sizeof(int));
+     memcpy(ds->data.chanmap.map, map, n_channels*sizeof(int));
+
+     return ds;
+}
+
 Datasource *datasource_new_silent(Dataformat *format, off_t samples)
 {
      Datasource *ds;
@@ -304,6 +360,7 @@ static gboolean datasource_open_sndfile(Datasource *ds)
      int fd;
      fd = xopen(ds->data.sndfile.filename,O_RDONLY,0);
      if (fd == -1) return TRUE;
+     i.format = 0;
      s = sf_open_fd(fd,SFM_READ,&i,TRUE);
      if (s == NULL) {	  
 	  d = g_strdup_printf(_("Couldn't open %s"),
@@ -317,8 +374,6 @@ static gboolean datasource_open_sndfile(Datasource *ds)
      ds->data.sndfile.handle = s;
      ds->data.sndfile.pos = 0;
 
-     sf_command ( s, SFC_SET_NORM_FLOAT, NULL, SF_TRUE );
-     sf_command ( s, SFC_SET_NORM_DOUBLE, NULL, SF_TRUE );
      return FALSE;
 #else
      g_assert_not_reached();
@@ -355,16 +410,33 @@ gboolean datasource_open(Datasource *ds)
 	  case DATASOURCE_BYTESWAP:	       
 	  case DATASOURCE_CONVERT:
 	       if (datasource_open(ds->data.clone)) return TRUE;
+	       break;
+	  case DATASOURCE_CHANMAP:
+	       if (datasource_open(ds->data.chanmap.clone)) return TRUE;
+	       break;
 	  }     
      ds->opencount ++;
      return FALSE;
 }
 
+static void clear_temparea(struct temparea *ta)
+{
+     if (ta->ptr != NULL) {
+	  g_free(ta->ptr);
+	  ta->ptr = NULL;
+	  ta->size = 0;	  
+     }
+}
+
 void datasource_close(Datasource *ds)
 {
      g_assert(ds->opencount != 0);
      ds->opencount --;
      if (ds->opencount > 0) return;
+
+     clear_temparea(&ds->read_temparea);
+     clear_temparea(&ds->readfp_temparea);
+
      switch (ds->type) {
      case DATASOURCE_VIRTUAL:
      case DATASOURCE_TEMPFILE:
@@ -380,6 +452,9 @@ void datasource_close(Datasource *ds)
      case DATASOURCE_CONVERT:
 	  datasource_close(ds->data.clone);
 	  break;
+     case DATASOURCE_CHANMAP:
+	  datasource_close(ds->data.chanmap.clone);
+	  break;
      }     
 }
 
@@ -403,7 +478,7 @@ static guint datasource_clone_read_array(Datasource *source, off_t sampleno,
      orig_adjust = orig_offset % source->data.clone->format.samplebytes;
      /* How much data should we read (should be able to fill buffer) */
      orig_size = size + orig_adjust + source->data.clone->format.samplebytes - 1;
-     p = g_malloc(orig_size);
+     p = datasource_get_temparea(&(source->read_temparea),orig_size);
      x = datasource_read_array(source->data.clone, orig_sampleno, orig_size, p,
 			       dither_mode,clipcount);
      if (x != 0) {
@@ -411,7 +486,6 @@ static guint datasource_clone_read_array(Datasource *source, off_t sampleno,
 	  memcpy(buffer, p+orig_adjust, size);
 	  x = size;
      }
-     g_free(p);
      return x;			       
 }
 
@@ -428,14 +502,13 @@ static void sndfile_read_error(sf_count_t x, Datasource *source)
 }
 #endif
 
-static guint datasource_sndfile_read_array_pcm(Datasource *source, 
-					       off_t sampleno, guint size,
-					       gpointer buffer)
+static guint datasource_sndfile_read_array(Datasource *source,
+					   off_t sampleno, guint size,
+					   gpointer buffer)
 {
 #if defined(HAVE_LIBSNDFILE)
      gchar c[256],*d;
-     sf_count_t x;
-     g_assert(source->format.type == DATAFORMAT_PCM);
+     sf_count_t samples,x;
      if (source->data.sndfile.pos != sampleno) {
 	  if (sf_seek(source->data.sndfile.handle,sampleno,SEEK_SET) == -1) {
 	       sf_error_str(source->data.sndfile.handle,c,sizeof(c));
@@ -447,67 +520,57 @@ static guint datasource_sndfile_read_array_pcm(Datasource *source,
 	  }
 	  source->data.sndfile.pos = sampleno;
      }
-     if (source->data.sndfile.raw_readable) {
+     samples = size / source->format.samplebytes;
+     if (source->format.type == DATAFORMAT_FLOAT) {
+	  x = sf_readf_sample_t(source->data.sndfile.handle,buffer,samples);
+     } else if (source->data.sndfile.raw_readable) {
 	  /* Read raw */
-	  x = sf_read_raw(source->data.sndfile.handle,buffer,size);	  
-	  if (x>0) source->data.sndfile.pos += x/source->format.samplebytes;
-	  if (x<size) {
-	       sndfile_read_error(x,source);
-	       return 0;
-	  }
-	  return x;
+	  x = sf_read_raw(source->data.sndfile.handle,buffer,size);
+	  x /= source->format.samplebytes;
      } else if (source->format.samplesize==2 && source->format.sign) {
 	  /* Read as short */
-	  x = sf_readf_short(source->data.sndfile.handle,buffer,
-			     size / source->format.samplebytes);
-	  if (x>0) source->data.sndfile.pos += x;
-	  if (x < size / source->format.samplebytes) {
-	       sndfile_read_error(x,source);
-	       return 0;
-	  }
-	  return x * source->format.samplebytes;
-     } else {	  
+	  x = sf_readf_short(source->data.sndfile.handle,buffer,samples);
+     } else if (source->format.samplesize==4 && source->format.sign &&
+		source->format.packing < 2) {
+	  /* Read as int */
+	  /* TODO: Handle archs with sizeof(int)>4 */
+	  x = sf_readf_int(source->data.sndfile.handle,buffer,samples);
+     } else {
 	  g_assert_not_reached();
 	  return 0;
      } 
+     if (x>0) source->data.sndfile.pos += x;
+     if (x<samples) {
+	  sndfile_read_error(x,source);
+	  return 0;		    
+     }
+     return x * source->format.samplebytes;
 #else
      g_assert_not_reached();
      return 0;
 #endif
 }
 
-static guint datasource_sndfile_read_array_fp(Datasource *source, 
-					      off_t sampleno, guint samples,
-					      sample_t *buffer)
+static void remap_main(void *source_buf, int item_size, int source_channels,
+		       void *dest_buf, int dest_channels, int *map, int items)
 {
-#if defined(HAVE_LIBSNDFILE)
-     gchar c[256],*d;
-     sf_count_t x;
-
-     if (source->data.sndfile.pos != sampleno) {
-	  if (sf_seek(source->data.sndfile.handle,sampleno,SEEK_SET) == -1) {
-	       sf_error_str(source->data.sndfile.handle,c,sizeof(c));
-	       d = g_strdup_printf(_("Error seeking in %s: %s"),
-				   source->data.sndfile.filename, c);
-	       user_error(d);
-	       g_free(d);
-	       return 0;
+     int i,j;
+     char *sp,*dp;
+     int dstep,sstep;
+     /* Simplest possible implementation - room for optimization */
+     dstep = item_size*dest_channels;
+     sstep = item_size*source_channels;
+     for (i=0; i<dest_channels; i++) {
+	  dp = ((char *)dest_buf) + i*item_size;
+	  if (map[i] < 0 || map[i] >= source_channels) {
+	       for (j=0; j<items; j++,dp+=dstep)
+		    memset(dp,0,item_size); 	       
+	  } else {
+	       sp = ((char *)source_buf) + map[i]*item_size;
+	       for (j=0; j<items; j++,sp+=sstep,dp+=dstep)
+		    memcpy(dp,sp,item_size);
 	  }
-	  source->data.sndfile.pos = sampleno;
      }
-
-     x = sf_readf_sample_t(source->data.sndfile.handle,buffer,samples);
-     if (x>0) source->data.sndfile.pos += x;
-     if (x<samples) {
-	  sndfile_read_error(x,source);
-	  return 0;		    
-     }
-     return x;
-
-#else
-     g_assert_not_reached();
-     return 0;
-#endif
 }
 
 static guint datasource_read_array_main(Datasource *source, 
@@ -516,7 +579,7 @@ static guint datasource_read_array_main(Datasource *source,
 					off_t *clipcount)
 {
      off_t x;
-     guint u;
+     guint u,s;
      sample_t *c;
      switch (source->type) {
      case DATASOURCE_REAL:
@@ -540,12 +603,7 @@ static guint datasource_read_array_main(Datasource *source,
 	  return size;
      case DATASOURCE_SNDFILE:
      case DATASOURCE_SNDFILE_TEMPORARY:
-	  if (source->format.type == DATAFORMAT_PCM)
-	       return datasource_sndfile_read_array_pcm(source,sampleno,size,
-							buffer);
-	  else
-	       return datasource_sndfile_read_array_fp
-		    (source,sampleno,size/source->format.samplebytes,buffer);
+	  return datasource_sndfile_read_array(source,sampleno,size,buffer);
      case DATASOURCE_REF:
 	  return datasource_read_array_main(source->data.clone,sampleno,size,
 					    buffer,dither_mode,clipcount);
@@ -559,23 +617,36 @@ static guint datasource_read_array_main(Datasource *source,
 	  return u;
      case DATASOURCE_CONVERT:
 	  u = size / source->format.samplebytes;
-	  if (source->format.type == DATAFORMAT_FLOAT && 
-	      source->format.samplesize == sizeof(sample_t)) 
+	  if (dataformat_samples_equal(&(source->format),&dataformat_sample_t))
 	       return datasource_read_array_fp(source->data.clone,sampleno,u,
 					       buffer,dither_mode,clipcount) *
 		    source->format.samplebytes ;
-	  c = g_malloc(u*sizeof(sample_t)*source->format.channels);
+	  c = (sample_t *)datasource_get_temparea(&(source->read_temparea), 
+						  u*sizeof(sample_t)*
+						  source->format.channels);
 	  u = datasource_read_array_fp(source->data.clone, sampleno, u, 
 				       (gpointer)c,dither_mode,clipcount);
 	  if (u > 0) {
-	       if (clipcount != NULL)
-		    *clipcount += 
-			 unnormalized_count(c,u*source->format.channels);
+	       apply_convert_factor(&(source->data.clone->format), &(source->format),
+				    (sample_t *)c, u*source->format.channels);
 	       convert_array(c,&dataformat_sample_t,buffer,&(source->format),
-			     u*source->format.channels,dither_mode);
+			     u*source->format.channels,dither_mode,clipcount);
 	  }
-	  g_free(c);
 	  return u * source->format.samplebytes;
+     case DATASOURCE_CHANMAP:	  
+	  u = (size / source->format.samplebytes) * 
+	       source->data.chanmap.clone->format.samplebytes;
+	  c = (sample_t *)datasource_get_temparea(&(source->read_temparea),u);
+	  g_assert(c != buffer);
+	  u = datasource_read_array_main(source->data.chanmap.clone, sampleno,
+					 u, c, dither_mode,clipcount);
+	  s = u / source->data.chanmap.clone->format.samplebytes;
+	  if (s > 0)
+	       remap_main(c, source->data.chanmap.clone->format.samplesize,
+			  source->data.chanmap.clone->format.channels,
+			  buffer, source->format.channels, 
+			  source->data.chanmap.map, s);
+	  return s*source->format.samplebytes;
      default:
 	  g_assert_not_reached();
 	  return 0;
@@ -617,6 +688,8 @@ guint datasource_read_array_fp(Datasource *source, off_t sampleno,
      gchar *p;
      guint x,s;
      
+     g_assert(source->opencount > 0);
+
      g_assert(sampleno <= source->length);
      if (samples > source->length-sampleno) 
 	  samples = (guint)(source->length-sampleno);
@@ -627,35 +700,32 @@ guint datasource_read_array_fp(Datasource *source, off_t sampleno,
      case DATASOURCE_SILENCE:
 	  memset(buffer,0,samples*source->format.channels*sizeof(sample_t));
 	  return samples;
-     case DATASOURCE_SNDFILE:
-     case DATASOURCE_SNDFILE_TEMPORARY:
-       return datasource_sndfile_read_array_fp(source,sampleno,samples,
-					       buffer);
      case DATASOURCE_REF:
      case DATASOURCE_CONVERT:
-	  return datasource_read_array_fp(source->data.clone,sampleno,
-					  samples,buffer,dither_mode,
-					  clipcount);
+	  g_assert(source->type == DATASOURCE_CONVERT ||
+		   dataformat_equal(&(source->format), &(source->data.clone->format)));
+	  x = datasource_read_array_fp(source->data.clone,sampleno,
+				       samples,buffer,dither_mode,
+				       clipcount);
+	  apply_convert_factor(&(source->data.clone->format), &(source->format),
+			       buffer, x*source->format.channels);
+	  return x;
      default:
-	  if (source->format.type == DATAFORMAT_FLOAT &&
-	      source->format.samplesize == sizeof(sample_t))
-	       
+	  if (dataformat_samples_equal(&(source->format),&dataformat_sample_t))
 	       return datasource_read_array(source,sampleno,
 					    samples*source->format.samplebytes,
 					    buffer, dither_mode, clipcount) 
 		    / source->format.samplebytes;
 	       
 	  s = samples * source->format.samplebytes;
-	  p = g_malloc(s);
+	  p = datasource_get_temparea(&(source->readfp_temparea),s);
 	  x = datasource_read_array(source,sampleno,s,p,dither_mode,clipcount);
 	  g_assert(x==s || x==0);
 	  if (x==s) {
 	       convert_array(p,&(source->format),buffer,&dataformat_sample_t,
-			     samples*source->format.channels,dither_mode);
-	       g_free(p);
+			     samples*source->format.channels,dither_mode,clipcount);
 	       return samples;
 	  }
-	  g_free(p);
 	  return 0;
      }
 }
@@ -767,7 +837,15 @@ Datasource *datasource_convert(Datasource *source, Dataformat *new_format)
      if (source->format.type == DATAFORMAT_PCM && 
 	 new_format->type == DATAFORMAT_PCM && 
 	 source->format.samplesize == new_format->samplesize &&
-	 source->format.sign == new_format->sign) {	  
+	 source->format.sign == new_format->sign &&
+	 source->format.packing == new_format->packing) {
+	  g_assert(XOR(source->format.bigendian,new_format->bigendian));
+	  return datasource_byteswap(source);
+     }
+     if (source->format.type == DATAFORMAT_FLOAT &&
+	 new_format->type == DATAFORMAT_FLOAT &&
+	 source->format.samplesize == new_format->samplesize) {
+	  g_assert(XOR(source->format.bigendian,new_format->bigendian));
 	  return datasource_byteswap(source);
      }
 
@@ -849,6 +927,8 @@ gint datasource_clip_check(Datasource *ds, StatusBar *bar)
      case DATASOURCE_REF:
      case DATASOURCE_CONVERT:
 	  return datasource_clip_check(ds->data.clone,bar);
+     case DATASOURCE_CHANMAP:
+	  return datasource_clip_check(ds->data.chanmap.clone,bar);
      default:
 	  break;
      }
