@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005 2006, Magnus Hjorth
+ * Copyright (C) 2005 2006 2007 2010 2011 2012, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #include <config.h>
@@ -40,6 +40,10 @@ static guint document_signals[LAST_SIGNAL] = { 0 };
 
 static GtkObjectClass *parent_class;
 
+static void document_set_cursor_main(Document *d, off_t cursorpos, 
+				     gboolean playslave, gboolean running,
+				     off_t bufpos);
+
 static void clear_marklist(struct MarkList *m)
 {
      int i;
@@ -292,6 +296,16 @@ gboolean document_save(Document *d, gchar *filename, gint type_id,
      return b;
 }
 
+static void cursor_cb(off_t pos, off_t bufpos, gboolean is_running)
+{     
+     Document *d = playing_document;
+     if (playing_document != NULL) {
+	  if (!is_running) playing_document=NULL; 
+	  document_set_cursor_main(d,pos,TRUE,is_running,bufpos);
+	  if (!is_running) gtk_object_unref(GTK_OBJECT(d));
+     }     
+}
+
 void document_play(Document *d, off_t start, off_t end, gboolean loop, 
 		   gfloat speed)
 {     
@@ -313,11 +327,19 @@ void document_play(Document *d, off_t start, off_t end, gboolean loop,
      } else {
 
 	  /* General case */
-	  if (player_play(d->chunk,start,end,loop)) return;
-	  if (playing_document != NULL)
-	       gtk_object_unref(GTK_OBJECT(playing_document));
+	  /* Stop the player if running, this should lead to cursor_cb
+	   * being called and clearing playing_document */
+	  player_stop();
+	  g_assert(playing_document == NULL);
+	  /* playing_document must be setup before calling player_play,
+	   * since for very short files the cursor_cb might be called
+	   * immediately with is_running==FALSE */
 	  playing_document = d;
 	  gtk_object_ref(GTK_OBJECT(d));
+	  if (player_play(d->chunk,start,end,loop,cursor_cb)) {
+	       playing_document = NULL;
+	       gtk_object_unref(GTK_OBJECT(d));
+	  }
      }
 
      /* This sets the speed to the proper value */
@@ -341,18 +363,14 @@ void document_play_selection(Document *d, gboolean loopmode, gfloat speed)
 
 void document_stop(Document *d, gboolean do_return)
 {
-     off_t o;
+     off_t s,e;
      g_assert(d != NULL);
      if (d != playing_document) return;
+     player_get_range(&s,&e);
      player_stop();
-     if (!do_return) {
-	  if (playing_document != NULL)
-	       gtk_object_unref(GTK_OBJECT(playing_document));
-	  playing_document = NULL;
-	  o = d->cursorpos;
-	  d->cursorpos = -1;
-	  document_set_cursor(d,o);
-     }
+     g_assert(playing_document == NULL);
+     if (do_return)
+	  document_set_cursor(d,s);     
 }
 
 /* Make sure the current state is stored properly in history */
@@ -593,7 +611,8 @@ gboolean document_apply_cb(Document *d, document_apply_proc proc,
 
 
 static void document_set_cursor_main(Document *d, off_t cursorpos, 
-				     gboolean running)
+				     gboolean playslave, gboolean running,
+				     off_t bufpos)
 {
      off_t vs,ve,dist;
 
@@ -601,13 +620,13 @@ static void document_set_cursor_main(Document *d, off_t cursorpos,
 
      if (d->cursorpos == cursorpos) return;
 
-     if (!running && playing_document == d && player_playing()) {
+     if (!playslave && playing_document == d && player_playing()) {
 	  player_set_buffer_pos(cursorpos);
 	  return;
      }
 
 	  
-     if (d->followmode) {
+     if (d->followmode && playslave) {
 	  dist = d->viewend - d->viewstart;
 	  if (view_follow_strict_flag) {
 	       vs = cursorpos - dist/2;
@@ -640,6 +659,8 @@ static void document_set_cursor_main(Document *d, off_t cursorpos,
      
      d->old_cursorpos = d->cursorpos;
      d->cursorpos = cursorpos;
+     d->old_playbufpos = d->playbufpos;
+     d->playbufpos = running ? bufpos : -1;
 
      gtk_signal_emit(GTK_OBJECT(d),
 		     document_signals[CURSOR_CHANGED_SIGNAL],
@@ -648,7 +669,7 @@ static void document_set_cursor_main(Document *d, off_t cursorpos,
 
 void document_set_cursor(Document *d, off_t cursorpos)
 {
-     document_set_cursor_main(d,cursorpos,FALSE);
+     document_set_cursor_main(d,cursorpos,FALSE,(playing_document == d && player_playing()),cursorpos);
 }
 
 off_t document_nudge_cursor(Document *d, off_t delta)
@@ -792,7 +813,7 @@ void document_set_mark(Document *d, gchar *label, off_t position)
 		     document_signals[STATE_CHANGED_SIGNAL]);
 }
 
-off_t document_get_mark(Document *d, gchar *label)
+off_t document_get_mark(Document *d, const gchar *label)
 {
      int i;
      for (i=0; i<d->marks.length; i++) {
@@ -862,39 +883,6 @@ void document_redo(Document *d)
      get_state_from_history(d);
 }
 
-
-gboolean document_update_cursors(void)
-{
-     off_t ui=0;
-     off_t s,e;
-     static GTimeVal last_time = { 0,0 };
-     GTimeVal tv,tv2;
-
-     if (playing_document == NULL) return FALSE;
-
-     if (!player_playing()) {
-	  player_get_range(&s,&e);
-	  if (s <= playing_document->chunk->length) 
-	       document_set_cursor_main(playing_document, s, FALSE);
-	  gtk_object_unref(GTK_OBJECT(playing_document));
-	  playing_document = NULL;
-	  return FALSE;
-     }
-
-     /* Make sure we don't do this to often.. more than 20
-      * times/second seems unneccesary waste of CPU.. */
-     g_get_current_time(&tv);
-     timeval_subtract(&tv2,&tv,&last_time);
-     if (tv2.tv_sec == 0 && tv2.tv_usec < 50000) return FALSE;
-     memcpy(&last_time,&tv,sizeof(last_time));
-
-     ui = player_get_real_pos();
-     if (ui != playing_document->cursorpos)
-	  document_set_cursor_main(playing_document, ui, TRUE);
-
-     return TRUE;
-}
-
 void document_set_status_bar(Document *d, StatusBar *bar)
 {
      d->bar = bar;
