@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005, Magnus Hjorth
+ * Copyright (C) 2005 2006, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -81,6 +81,7 @@ typedef struct {
      off_t old_selstart, old_selend;
      /* Used when sending the cursor_changed signal */
      off_t old_cursorpos;
+     off_t playbufpos, old_playbufpos;
 
      gboolean followmode;       /* Follow mode flag */     
 
@@ -230,7 +231,7 @@ void document_set_mark(Document *d, gchar *label, off_t position);
  * if it doesn't exist.
  */
 
-off_t document_get_mark(Document *d, gchar *label);
+off_t document_get_mark(Document *d, const gchar *label);
 #define DOCUMENT_BAD_MARK ((off_t)-1)
 
 
@@ -252,6 +253,4 @@ void document_undo(Document *d);
 gboolean document_can_redo(Document *d);
 void document_redo(Document *d);
 
-gboolean document_update_cursors(void);
-
 #endif
