@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2009, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #include <config.h>
@@ -30,12 +30,10 @@ static gboolean updating = FALSE;
 
 static void document_list_changed(Combo *combo)
 {
-     gint s;
      Document *d=NULL;
      DocumentList *dl = DOCUMENT_LIST(combo);
 
      if (updating) return;
-     s = list_object_get_size(document_objects);
      d = DOCUMENT(list_object_get(document_objects,
 				  combo_selected_index(combo)));     
      if (d != dl->selected) {
@@ -91,6 +89,7 @@ static void document_list_init(GtkObject *obj)
 {
      DocumentList *mwl = DOCUMENT_LIST(obj);
      /* Most initialization is done in document_list_setup */
+     combo_set_max_request_width(COMBO(obj),350);
      mwl->selected = NULL;
      gtk_signal_connect_while_alive(GTK_OBJECT(document_objects), 
 				    "item_added",
@@ -146,29 +145,43 @@ GtkWidget *document_list_new(Document *chosen)
      return widget;     
 }
 
+struct setup_func_data {
+     GList *lp;
+     Document *first;
+};
+
 static void document_list_setup_func(gpointer item, gpointer user_data)
 {
-     GList **lp = (GList **)user_data;
+     struct setup_func_data *sfdp = (struct setup_func_data *)user_data;
      Document *w = DOCUMENT(item);
-     if (w->titlename != NULL)
-	  *lp = g_list_append(*lp, w->titlename);
+     if (w->titlename != NULL) {
+	  sfdp->lp = g_list_append(sfdp->lp, w->titlename);
+	  if (sfdp->first == NULL) sfdp->first = w;
+     }
 }
 
 void document_list_setup(DocumentList *mwl, Document *chosen)
 {   
-     GList *l = NULL;
      gint i;
+     struct setup_func_data sfd;
      updating = TRUE;     
-     list_object_foreach(document_objects,document_list_setup_func,&l);
-     combo_set_items(COMBO(mwl),l,0);
+
+     sfd.lp = NULL;
+     sfd.first = NULL;
+     list_object_foreach(document_objects,document_list_setup_func,&sfd);
+     combo_set_items(COMBO(mwl),sfd.lp,0);
+
+     if (chosen == NULL) 
+	  chosen = sfd.first;
+
      if (chosen) {
-	  i = g_list_index(l,chosen->titlename);
+	  i = g_list_index(sfd.lp,chosen->titlename);
 	  g_assert(i >= 0);
 	  combo_set_selection(COMBO(mwl),i);
 	  memcpy(&(mwl->format),&(chosen->chunk->format),
 		 sizeof(Dataformat));
 	  mwl->selected = chosen;
      }
-     g_list_free(l);     
+     g_list_free(sfd.lp);     
      updating = FALSE;
 }
