@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 2004 2007, Magnus Hjorth
+ * Copyright (C) 2003 2004 2007 2009 2011, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -35,6 +35,7 @@
 #include "um.h"
 #include "gettext.h"
 #include "mapchannelsdialog.h"
+#include "sandwichdialog.h"
 
 struct source {
      gchar tag;
@@ -108,6 +109,8 @@ static void builtin_rebuild_func(gchar source_tag, gpointer user_data)
 				author,loc);
      effect_register_add_effect(source_tag,"combine",_("Combine channels"),
 				author,loc);
+     effect_register_add_effect(source_tag,"sandwich",
+				_("Add channels from other file"),author,loc);
      effect_register_add_effect(source_tag,"speed",_("Speed"),author,loc);
      effect_register_add_effect(source_tag,"pipe",_("Pipe through program"),
 				author,loc);
@@ -126,6 +129,7 @@ static EffectDialog *builtin_get_func(gchar *name, gchar source_tag,
 	  type = combine_channels_dialog_get_type();
      else if (!strcmp(name,"speed")) type = speed_dialog_get_type();
      else if (!strcmp(name,"pipe")) type = pipe_dialog_get_type();
+     else if (!strcmp(name,"sandwich")) type = sandwich_dialog_get_type();
      if (type >= 0) 
 	  return EFFECT_DIALOG(gtk_type_new(type));
      else
@@ -795,8 +799,6 @@ static void effect_browser_init(EffectBrowser *eb)
 #else
      b251 = gtk_button_new_with_label(_("Apply"));
 #endif
-     gtk_widget_add_accelerator (b251, "clicked", ag, GDK_KP_Enter, 0, (GtkAccelFlags) 0);
-     gtk_widget_add_accelerator (b251, "clicked", ag, GDK_Return, 0, (GtkAccelFlags) 0);
      gtk_signal_connect(GTK_OBJECT(b251),"clicked",(GtkSignalFunc)apply_click,
 			eb);
 
@@ -847,6 +849,8 @@ static void effect_browser_init(EffectBrowser *eb)
      }
      gtk_container_set_border_width(GTK_CONTAINER(eb),5);
      gtk_container_add(GTK_CONTAINER(eb),b);
+     GTK_WIDGET_SET_FLAGS(GTK_WIDGET(b251),GTK_CAN_DEFAULT);
+     gtk_widget_grab_default(GTK_WIDGET(b251));
      gtk_widget_show_all(b);
 }
 
