@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006 2011, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #include <config.h>
@@ -30,6 +30,7 @@
 #include "um.h"
 #include "gtkfiles.h"
 #include "main.h"
+#include "mainloop.h"
 #include "inifile.h"
 #include "gettext.h"
 
@@ -388,7 +389,7 @@ gchar *get_filename(gchar *current_name, gchar *filemask, gchar *title_text,
 
 
      get_filename_quitflag=FALSE;
-     while (!get_filename_quitflag) mainloop(TRUE);
+     while (!get_filename_quitflag) mainloop();
      if (get_filename_result != NULL)
 	  cleanup_filename(get_filename_result);
      return get_filename_result;     
@@ -444,7 +445,7 @@ gchar *get_directory(gchar *current_name, gchar *title_text)
      gtk_widget_show(GTK_WIDGET(f));
 
      get_filename_quitflag=FALSE;
-     while (!get_filename_quitflag) mainloop(TRUE);
+     while (!get_filename_quitflag) mainloop();
      if (get_filename_result != NULL) cleanup_filename(get_filename_result);
      return get_filename_result;     
      
@@ -548,7 +549,7 @@ static gchar *get_filename_main(gchar *current_name, gchar *title_text,
      gtk_signal_connect(GTK_OBJECT(fc),"response",GTK_SIGNAL_FUNC(response),
 			&sr);
      gtk_widget_show(w);
-     while (!sr.responded) mainloop(TRUE); 
+     while (!sr.responded) mainloop(); 
      c = NULL;
      if (sr.r == GTK_RESPONSE_ACCEPT) 
 	  c = gtk_file_chooser_get_filename(fc);
@@ -796,3 +797,38 @@ gboolean fd_canread(int fd)
      else return TRUE;
 }
 
+gboolean e_fread_u16_xe(guint16 *data, EFILE *stream, gboolean be)
+{
+     unsigned char buf[2];
+     guint16 i;
+     if (e_fread(buf,2,stream)) return TRUE;
+     if (be) { 
+	  i=buf[0];
+	  i<<=8; i|=buf[1];
+     } else {
+	  i=buf[1];
+	  i<<=8; i|=buf[0];
+     }
+     *data = i;
+     return FALSE;
+}
+
+gboolean e_fread_u32_xe(guint32 *data, EFILE *stream, gboolean be)
+{
+     unsigned char buf[4];
+     guint32 i;
+     if (e_fread(buf,4,stream)) return TRUE;
+     if (be) {
+	  i=buf[0];
+	  i<<=8; i|=buf[1];
+	  i<<=8; i|=buf[2];
+	  i<<=8; i|=buf[3];
+     } else {
+	  i=buf[3];
+	  i<<=8; i|=buf[2];
+	  i<<=8; i|=buf[1];
+	  i<<=8; i|=buf[0];
+     }
+     *data = i;
+     return FALSE;
+}
