@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, Magnus Hjorth
+ * Copyright (C) 2004 2009, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #include <config.h>
@@ -92,6 +92,18 @@ ListObject *list_object_new(gboolean do_ref)
      return lo;
 }
 
+ListObject *list_object_new_from_list(GList *list, gboolean do_ref)
+{
+     ListObject *lo = gtk_type_new(list_object_get_type());
+     lo->do_ref = do_ref;
+     lo->list = list;
+     if (do_ref) {
+	  g_list_foreach(list,(GFunc)gtk_object_ref,NULL);
+	  g_list_foreach(list,(GFunc)gtk_object_sink,NULL);
+     }
+     return lo;
+}
+
 void list_object_add(ListObject *lo, gpointer ptr)
 {
      lo->list = g_list_append(lo->list, ptr);
