@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005 2007, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2007 2008 2010 2011 2012, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -29,7 +29,9 @@
 #include <errno.h>
 #include <locale.h>
 #include <signal.h>
+#include <ctype.h>
 #include <gtk/gtk.h>
+#include "mainloop.h"
 #include "mainwindow.h"
 #include "datasource.h"
 #include "player.h"
@@ -50,7 +52,6 @@
 #endif
 
 GdkPixmap *icon = NULL;
-gboolean idle_work_flag;
 gboolean quitflag;
 gboolean quality_mode = TRUE;
 gchar *driver_option = NULL;
@@ -68,24 +69,55 @@ const char *strip_context(const char *s)
      return s;
 }
 
-void mainloop(gboolean force_sleep)
+static int idle_work(gpointer csource, gpointer user_data)
 {
-     static guint player_count=0;
-     if (player_work()) { player_count=0; return; }
-     player_count++;
-     if (gtk_events_pending()) {
-	  gtk_main_iteration();
-	  return;
-     }
-     idle_work_flag = TRUE;
-     if (status_bar_progress_count()>0 && !force_sleep) return;
-     document_update_cursors();
-     if (chunk_view_autoscroll() && !force_sleep) return;
-     if (mainwindow_update_caches() && !force_sleep) return;
-     if (player_count > 10)
-	  do_yield(TRUE);
+     if (!idle_work_flag) {
+	  idle_work_flag = TRUE;
+	  return 1;
+     }     
+     if (status_bar_progress_count() > 0) return 1;
+     if (mainwindow_update_caches()) return 1;
+     return -1;
 }
 
+#if GTK_MAJOR_VERSION == 1
+
+void gdk_gc_set_rgb_fg_color(GdkGC *gc, GdkColor *clr)
+{
+     static GdkColor cached = {0,-1,-1,-1};
+     if (clr->red == cached.red && clr->green == cached.green && 
+	 clr->blue == cached.blue) 
+	  clr->pixel = cached.pixel;
+     else {
+	  gdk_colormap_alloc_color(gdk_colormap_get_system(), 
+				   clr,FALSE,TRUE);
+	  memcpy(&cached,clr,sizeof(cached));
+     }
+     gdk_gc_set_foreground(gc, clr);
+}
+
+#endif
+
+#if GTK_MAJOR_VERSION == 1 || (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 18)
+
+void gtk_widget_set_has_window(GtkWidget *w, gboolean has_window)
+{
+     if (!has_window)
+	  GTK_WIDGET_SET_FLAGS(w,GTK_NO_WINDOW);
+     else
+	  GTK_WIDGET_UNSET_FLAGS(w,GTK_NO_WINDOW);
+}
+
+void gtk_widget_set_can_focus(GtkWidget *w, gboolean can_focus)
+{
+     if (!can_focus)
+	  GTK_WIDGET_SET_FLAGS(w,GTK_CAN_FOCUS);
+     else
+	  GTK_WIDGET_UNSET_FLAGS(w,GTK_CAN_FOCUS);
+}
+
+#endif
+
 int main(int argc, char **argv)
 {
      int i;
@@ -179,16 +211,19 @@ int main(int argc, char **argv)
      autoplay_mark_flag = inifile_get_gboolean("autoPlayMark",FALSE);
      varispeed_reset_flag = inifile_get_gboolean("speedReset",FALSE);
      varispeed_smooth_flag = inifile_get_gboolean("speedSmooth",TRUE);
-     dataformat_get_from_inifile("playerFallback",FALSE,
+     dataformat_get_from_inifile("playerFallback",TRUE,
 				 &player_fallback_format);
      chunk_filter_use_floating_tempfiles = 
 	  inifile_get_gboolean("tempfilesFP",TRUE);
-     dither_editing = inifile_get_guint32("ditherEditing",DITHER_TRIANGULAR);
+     dither_editing = inifile_get_guint32("ditherEditing",DITHER_MINIMAL);
      if (dither_editing < 0 || dither_editing > DITHER_MAX)
-	  dither_editing = DITHER_TRIANGULAR;
+	  dither_editing = DITHER_MINIMAL;
      dither_playback = inifile_get_gboolean("ditherPlayback",DITHER_NONE);
      if (dither_playback < 0 || dither_playback > DITHER_MAX)
 	  dither_playback = DITHER_NONE;
+     sample_convert_mode = inifile_get_guint32("sampleMode",CONVERT_MODE_NOOFFS);
+     if (sample_convert_mode < 0 || sample_convert_mode > CONVERT_MODE_MAX)
+	  sample_convert_mode = CONVERT_MODE_NOOFFS;
 
      gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
 
@@ -225,10 +260,12 @@ int main(int argc, char **argv)
 
      /* gtk_idle_add(idle_work,NULL); */
 
+     /* Add low priority idle function */
+     mainloop_constant_source_add(idle_work,NULL,TRUE);
 
      /* Run it! */
      while (!quitflag)
-	  mainloop(FALSE);
+	  mainloop();
 
 
 
@@ -323,7 +360,7 @@ GdkColor factory_default_colors[] = {
      { 0, 0, 0, 0 }, { 0, 29952, 50176, 27392 }, { 0, 41216, 39936, 52992 },
      { 0, 62720, 62720, 62720 }, { 0, 60160, 55552, 0 },
      { 0, 43776, 11776, 0 }, { 0, 0xff00, 0xff00, 0xff00 },
-     { 0, 0x8000, 0x8000, 0x8000 }
+     { 0, 0x8000, 0x8000, 0x8000 }, { 0, 0x6200, 0x7c00, 0x5e00 }
 };
 static GdkGC *gc_table[LAST_COLOR] = { };
 
@@ -338,13 +375,14 @@ gchar *color_names[] = {
     N_("Selection"),
     N_("Progress bar"),
     N_("Zero-level"),
+    N_("Buffer position")
 };
 
 gchar *color_inifile_entry[] = { NULL, NULL, "colorBG", 
 				 "colorWave1", "colorWave2",
 				 "colorCursor","colorMark",
 				 "colorSelection","colorProgress",
-				 "colorBars"
+				 "colorBars","colorBufpos"
 };
 
 GdkColor *get_color(enum Color c)
@@ -398,11 +436,11 @@ static void parse_color(gchar *str, GdkColor *color)
 void set_custom_colors(GdkColor *c)
 {
      gint i;
-     gchar *d,*e;
+     gchar *d;
      if (!c) {
 	  set_custom_colors(factory_default_colors);
 	  for (i=FIRST_CUSTOM_COLOR; i<LAST_COLOR; i++) {
-	       e = d = inifile_get(color_inifile_entry[i],NULL);
+	       d = inifile_get(color_inifile_entry[i],NULL);
 	       /* printf("%s -> %s\n",color_inifile_entry[i],d); */
 	       if (d != NULL) {
 		    /* printf("Before: %x,%x,%x\n",color_table[i].red,
@@ -487,7 +525,7 @@ void byteswap(void *buffer, int element_size, int buffer_size)
      int i,j;
      char *c,*d,*bufend=NULL;
      char tempbuf[BYTESWAP_BSIZE];
-     g_assert(element_size < 7);
+     g_assert(element_size < 7 || element_size==8);
      if (element_size == 1) return;
      while (buffer_size != 0) {
 	  i = BYTESWAP_BSIZE;
@@ -551,7 +589,7 @@ gchar *get_time(guint32 samplerate, off_t samples, off_t samplemax,
 		gchar *timebuf, gint mode)
 {
      static gchar static_buf[64];
-     gfloat secs, ffps, fframes;
+     gfloat secs, ffps;
      guint mins, msecs, hours, maxhours, frames, ifps, isecs;
      guint fptm;
 
@@ -563,7 +601,7 @@ gchar *get_time(guint32 samplerate, off_t samples, off_t samplemax,
      if (mode > 6) mode = 0;
 
      if (mode == 2) {
-	  g_snprintf(timebuf,50,"%05" OFF_T_FORMAT,samples);
+	  g_snprintf(timebuf,50,"%05" OFF_T_FORMAT,(OFF_T_FTYPE)samples);
      } else if (mode < 2) {
 	  secs = (gfloat) samples / (gfloat) samplerate;
 	  mins = (guint) (secs / 60.0);
@@ -601,7 +639,6 @@ gchar *get_time(guint32 samplerate, off_t samples, off_t samplemax,
 	  else if (mode == 5) { ffps=30.0*1.000/1.001; ifps=30; }
 	  else { ffps=30.0; ifps=30; }
 
-	  fframes = secs * ffps;
 	  frames = (guint) (secs * ffps);
 
 	  /* Ten-minute units never need to be dropped. */
@@ -1149,3 +1186,15 @@ char *translate_strip(const char *s)
      } else
 	  return c;
 }
+
+void format_float(float f, char *r, int maxsz)
+{
+     gchar *c;
+     g_snprintf(r,maxsz,"%#.7g",f);
+     c = strchr(r,0);
+     c-=2;
+     while (c>=r && isdigit(c[0]) && c[1]=='0') {
+	  c[1] = 0;
+	  c--;
+     }     
+}
