@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2002 2003 2004 2005 2006 2007, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006 2007 2008 2009 2010, Magnus Hjorth
+ * Copyright (C) 2011 2012 2013 2018, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -44,6 +45,7 @@
 #include "help.h"
 #include "ladspadialog.h"
 #include "gettext.h"
+#include "bitmap.h"
 
 #include "button_open.xpm"
 #include "button_save.xpm"
@@ -65,12 +67,28 @@
 #include "button_mixer.xpm"
 #include "button_bounce.xpm"
 #include "icon.xpm"
-#include "vzoom.xpm"
-#include "hzoom.xpm"
-#include "speed.xpm"
+#include "vzoom.xbm"
+#include "hzoom.xbm"
+#include "speed.xbm"
 
 /* #define SHOW_DEBUG_MENU */
 
+#if GTK_MAJOR_VERSION < 2
+#define INV_SPEED
+#endif
+
+#ifdef FIXED_DATE
+#define BUILD_DATE FIXED_DATE
+#else
+#define BUILD_DATE __DATE__
+#endif
+
+#ifdef FIXED_TIME
+#define BUILD_TIME FIXED_TIME
+#else
+#define BUILD_TIME __TIME__
+#endif
+
 ListObject *mainwindow_objects = NULL;
 
 static gboolean window_geometry_stack_inited = FALSE;
@@ -81,8 +99,8 @@ static Chunk *clipboard = NULL;
 gboolean autoplay_mark_flag = FALSE;
 gboolean varispeed_reset_flag = FALSE;
 
-static void mainwindow_set_document(Mainwindow *w, Document *doc, 
-				    gchar *filename);
+static Mainwindow *mainwindow_set_document(Mainwindow *w, Document *doc,
+					   gchar *filename);
 static void mainwindow_view_changed(Document *d, gpointer user_data);
 static void mainwindow_selection_changed(Document *d, gpointer user_data);
 static void mainwindow_cursor_changed(Document *d, gboolean rolling, 
@@ -175,8 +193,12 @@ static void procstart(StatusBar *bar, gpointer user_data)
      Mainwindow *w = MAINWINDOW(user_data);
      mainwindow_set_sensitive(w,FALSE);
      w->esc_pressed_flag = FALSE;
+#if GTK_MAJOR_VERSION > 1
+     gtk_window_present(GTK_WINDOW(w));
+#else
      if (GTK_WIDGET(w)->window)
 	  gdk_window_raise(GTK_WIDGET(w)->window);
+#endif
      gtk_grab_add(GTK_WIDGET(w));
 }
 
@@ -192,27 +214,13 @@ static void fix_title(Mainwindow *wnd)
 {
      gchar *c;
      if (wnd->doc != NULL) {
-	  if (wnd->doc->chunk->format.type == DATAFORMAT_FLOAT)
-	       
-	       c = g_strdup_printf ( _("mhWaveEdit: %s (%s): %d Hz, "
-				     "%s"), 
-				     wnd->doc->titlename, 
-				     chunk_get_time(wnd->doc->chunk,
-						    wnd->doc->chunk->length,
-						    NULL),
-				     wnd->doc->chunk->format.samplerate,
-				     (wnd->doc->chunk->format.samplesize >
-				      sizeof(float))?_("double"):_("float"));
-	  else
-	       c = g_strdup_printf ( _("mhWaveEdit: %s (%s): %d Hz, "
-				     "%d bit"), 
-				     wnd->doc->titlename, 
-				     chunk_get_time(wnd->doc->chunk,
-						    wnd->doc->chunk->length,
-						    NULL),
-				     wnd->doc->chunk->format.samplerate,
-				     wnd->doc->chunk->format.samplesize*8);
-
+	  c = g_strdup_printf ( _("mhWaveEdit: %s (%s): %d Hz, %s"),
+				wnd->doc->titlename,
+				chunk_get_time(wnd->doc->chunk,
+					       wnd->doc->chunk->length,
+					       NULL),
+				wnd->doc->chunk->format.samplerate,
+				sampletype_name(&(wnd->doc->chunk->format)));
      } else 
 	  c = g_strdup(PROGRAM_VERSION_STRING);
      gtk_window_set_title ( GTK_WINDOW ( wnd ), c );
@@ -463,11 +471,15 @@ static void do_play(Mainwindow *w, off_t start, off_t end, gboolean loop)
      if (varispeed_reset_flag && 
 	 !(player_playing() && playing_document == w->doc))
 	  speed_reset = TRUE;
-     if (speed_reset) {
-	  document_stop(w->doc,FALSE);
-	  gtk_adjustment_set_value(w->speed_adj,-1.0);
-     }
-     document_play(w->doc, start, end, loop, -w->speed_adj->value);
+     if (speed_reset) document_stop(w->doc,FALSE);
+#ifdef INV_SPEED
+     if (speed_reset) gtk_adjustment_set_value(w->speed_adj,-1.0);
+     document_play(w->doc, start, end, loop, -w->speed_adj->value);     
+#else
+     if (speed_reset) gtk_adjustment_set_value(w->speed_adj,1.0);
+     document_play(w->doc, start, end, loop, w->speed_adj->value);
+#endif
+     update_desc(w);
 }
 
 
@@ -535,6 +547,9 @@ static gint mainwindow_keypress(GtkWidget *widget, GdkEventKey *event)
 		    document_set_cursor(w->doc,o);
 	       }
 	       return TRUE;
+	  case GDK_Tab:
+	       o = (w->doc->viewstart + w->doc->viewend) / 2;
+	       document_set_cursor(w->doc,o);
 	  }
      else 
 	  switch (event->keyval) {	       
@@ -567,7 +582,9 @@ static gint mainwindow_keypress(GtkWidget *widget, GdkEventKey *event)
 	  case GDK_period: edit_stop(NULL,widget); return TRUE;
 	  case GDK_slash: edit_playselection(NULL, widget); return TRUE;
 	  case GDK_space:
-	       if (playing_document == w->doc) 
+	       if ((event->state & GDK_SHIFT_MASK) != 0)
+		    do_play(w,0,w->doc->chunk->length,w->loopmode);
+	       else if (playing_document == w->doc) 
 		    document_stop(w->doc,w->bouncemode);
 	       else if (w->doc != NULL)
 		    do_play(w,w->doc->cursorpos,w->doc->chunk->length,
@@ -575,13 +592,19 @@ static gint mainwindow_keypress(GtkWidget *widget, GdkEventKey *event)
 	       return TRUE;	       
 	  case GDK_Left:
 	  case GDK_KP_Left:
-	       document_scroll(w->doc, 
-			       -(w->doc->viewend - w->doc->viewstart)/MAINWINDOW_SCROLL_DELTA_RATIO);
+	       if (playing_document==w->doc && w->doc->followmode)
+		    player_nudge(-0.5);
+	       else
+		    document_scroll(w->doc,
+				    -(w->doc->viewend - w->doc->viewstart)/MAINWINDOW_SCROLL_DELTA_RATIO);
 	       return TRUE;
 	  case GDK_Right:
 	  case GDK_KP_Right:
-	       document_scroll(w->doc,
-			       (w->doc->viewend - w->doc->viewstart)/MAINWINDOW_SCROLL_DELTA_RATIO);
+	       if (playing_document==w->doc && w->doc->followmode)
+		    player_nudge(+0.5);
+	       else
+		    document_scroll(w->doc,
+				    (w->doc->viewend - w->doc->viewstart)/MAINWINDOW_SCROLL_DELTA_RATIO);
 	       return TRUE;
 	  case GDK_parenleft:
 	       o = 3*w->doc->chunk->format.samplerate;
@@ -608,6 +631,23 @@ static gint mainwindow_keypress(GtkWidget *widget, GdkEventKey *event)
 	       else
 		    do_play(w,w->doc->selend-o,w->doc->selend,FALSE);
 	       return TRUE;		    
+	  case GDK_Home:
+	       document_set_view(w->doc,0,w->doc->viewend-w->doc->viewstart);
+	       if (playing_document != w->doc || w->doc->followmode)
+		    document_set_cursor(w->doc,0);
+	       return TRUE;
+	  case GDK_End:
+	       /* Stop playback to prevent cursor jumping back to start point */
+	       document_stop(w->doc,FALSE);
+	       document_scroll(w->doc,w->doc->chunk->length);
+	       if (playing_document != w->doc || w->doc->followmode)
+		    document_set_cursor(w->doc,w->doc->chunk->length);
+	       return TRUE;
+	  case GDK_Tab:
+	       document_scroll(w->doc,
+			       w->doc->cursorpos-
+			       (w->doc->viewend+w->doc->viewstart)/2);
+	       return TRUE;
 	  }
      return GTK_WIDGET_CLASS(parent_class)->key_press_event(widget,event);
 }
@@ -696,7 +736,7 @@ static void mainwindow_class_init(GtkObjectClass *klass)
 	  mainwindow_drag_data_received;
 }
 
-static void mainwindow_set_document(Mainwindow *w, Document *d, 
+static Mainwindow *mainwindow_set_document(Mainwindow *w, Document *d,
 				    gchar *filename)
 {
      if (w->doc != NULL) {
@@ -724,13 +764,14 @@ static void mainwindow_set_document(Mainwindow *w, Document *d,
 	  gtk_widget_set_sensitive(w->speed_slider,FALSE);
      list_object_add(mainwindow_objects,w);
      mainwindow_view_changed(w->doc,w);
+     return w;
 }
 
-static void mainwindow_set_chunk(Mainwindow *w, Chunk *c, gchar *filename)
+static Mainwindow *mainwindow_set_chunk(Mainwindow *w, Chunk *c, gchar *filename)
 {
      Document *d;
      d = document_new_with_chunk(c,filename,w->statusbar);
-     mainwindow_set_document(w,d,filename);
+     return mainwindow_set_document(w,d,filename);
 }
 
 void mainwindow_set_speed_sensitive(gboolean sensitive)
@@ -803,6 +844,7 @@ static void file_close(GtkMenuItem *menuitem, gpointer user_data)
 	  fix_title(w);
 	  set_sensitive(w->need_chunk_items,FALSE);
 	  set_sensitive(w->need_undo_items,FALSE);
+	  set_sensitive(w->need_selection_items,FALSE);
 	  list_object_remove(mainwindow_objects,w);
      } else {
 	  geometry_stack_push(GTK_WINDOW(w),NULL,&window_geometry_stack);
@@ -850,7 +892,7 @@ static void edit_redo(GtkMenuItem *menuitem, gpointer user_data)
 
 static void update_clipboard_items(Mainwindow *w, gpointer user_data)
 {
-     set_sensitive(w->need_clipboard_items,(int)user_data);
+     set_sensitive(w->need_clipboard_items,user_data != NULL);
 }
 
 static void edit_cut(GtkMenuItem *menuitem, gpointer user_data)
@@ -869,7 +911,7 @@ static void edit_cut(GtkMenuItem *menuitem, gpointer user_data)
 
      document_update(w->doc, chunk, w->doc->selstart, -(clipboard->length));
      list_object_foreach(mainwindow_objects, (GFunc)update_clipboard_items, 
-			 (gpointer)1);
+			 (gpointer)w);
 }
 
 static void edit_crop(GtkMenuItem *menu_item, gpointer user_data)
@@ -892,14 +934,14 @@ static void edit_copy(GtkMenuItem *menu_item, gpointer user_data)
      gtk_object_sink(GTK_OBJECT(clipboard));
 
      list_object_foreach(mainwindow_objects, (GFunc)update_clipboard_items, 
-			 (gpointer)1);
+			 (gpointer)w);
 }
 
 static void edit_paste(GtkMenuItem *menu_item, gpointer user_data)
 {
      Chunk *c = clipboard, *nc;
      Mainwindow *w = MAINWINDOW(user_data);
-     off_t cp;
+     off_t cp,cl;
      if (w->doc == NULL) {
 	  mainwindow_set_chunk(w,clipboard,NULL);
 	  return;
@@ -908,22 +950,22 @@ static void edit_paste(GtkMenuItem *menu_item, gpointer user_data)
      if (!dataformat_equal(&(w->doc->chunk->format),&(c->format))) {
 	  c = chunk_convert(c,&(w->doc->chunk->format),dither_editing,
 			    w->statusbar);
-	  if (c == NULL) return;
+	  if (c == NULL) return;	  
      }
-
-     nc = chunk_insert(w->doc->chunk,c,w->doc->cursorpos);
+     cl = c->length;
+     nc = chunk_insert(w->doc->chunk,c,w->doc->cursorpos);     
      gtk_object_sink(GTK_OBJECT(c));
 
      cp = w->doc->cursorpos;
-     document_update(w->doc, nc, cp, clipboard->length);
-     document_set_selection( w->doc, cp, cp + clipboard->length );
+     document_update(w->doc, nc, cp, cl);
+     document_set_selection( w->doc, cp, cp + cl );
 }
 
 static void edit_pasteover(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW(user_data);
      Chunk *c,*d=clipboard;
-     off_t orig_len;
+     off_t orig_len,dl;
      if (w->doc == NULL) {
 	  mainwindow_set_chunk(w,clipboard,NULL);
 	  return;
@@ -934,15 +976,16 @@ static void edit_pasteover(GtkMenuItem *menuitem, gpointer user_data)
 			    w->statusbar);
 	  if (d == NULL) return;
      }
+     dl = d->length;
 
-     orig_len = MIN(w->doc->chunk->length-w->doc->cursorpos, d->length);
+     orig_len = MIN(w->doc->chunk->length-w->doc->cursorpos, dl);
      c = chunk_replace_part(w->doc->chunk, w->doc->cursorpos, 
 			    orig_len, d);
      gtk_object_sink(GTK_OBJECT(d));
 
      document_update(w->doc, c, 0, 0);
      document_set_selection( w->doc, w->doc->cursorpos,
-			     w->doc->cursorpos + clipboard->length );
+			     w->doc->cursorpos + dl );
 }
 
 static void edit_mixpaste(GtkMenuItem *menuitem, gpointer user_data)
@@ -1061,6 +1104,12 @@ static void debug_dummy(GtkMenuItem *menuitem, gpointer user_data)
      document_apply(w->doc,dummy_proc,NULL,CHUNK_FILTER_MANY,FALSE,"DUMMY");
 }
 
+static void debug_dummy2(GtkMenuItem *menuitem, gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     document_apply(w->doc,dummy_proc,NULL,CHUNK_FILTER_MANY,TRUE,"DUMMY");
+}
+
 static void checkoc_proc(Chunk *chunk, gpointer user_data)
 {
      if (chunk->opencount > 0) 
@@ -1077,7 +1126,7 @@ static void dump_format(Dataformat *df)
 {
      printf("%d Channels @ %dHz ",df->channels,df->samplerate);
      if (df->type == DATAFORMAT_FLOAT) {
-	  if (df->samplesize > sizeof(float)) {
+	  if (df->samplesize <= sizeof(float)) {
 	       puts("float");
 	  } else
 	       puts("double");
@@ -1212,7 +1261,7 @@ static void effects_mixchannels(GtkMenuItem *menuitem, gpointer user_data)
      }
 }
 
-static void effects_splitchannel(GtkMenuItem *menuitem, gboolean user_data)
+static void effects_splitchannel(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW (user_data);
      Chunk *c;
@@ -1272,14 +1321,40 @@ static void edit_playall(GtkMenuItem *menuitem, gpointer user_data)
      do_play(w,0,w->doc->chunk->length,w->loopmode);
 }
 
-static void help_readme(void)
+static gchar *get_help_page_contents(int page)
+{
+     const char **c;
+     char *ts[128];
+     int i,j,k,l,x;
+     gchar *r;
+     c = help_page_contents[page];
+     l = 0;
+     for (i=0; c[i]!=NULL && i<128; i++) {
+	  ts[i] = gettext(c[i]);
+	  l += strlen(ts[i]);
+     }
+     r = g_malloc(l+1);
+     k = 0;
+     for (j=0; j<i; j++) {
+	  x = strlen(ts[j]);
+	  memcpy(r+k, ts[j], x);
+	  k += x;
+     }
+     r[k] = 0;
+     g_assert(k==l);
+     return r;
+}
+
+static void help_readme(GtkMenuItem *menuitem, gpointer user_data)
 {
      GtkAccelGroup* ag;
      GtkWidget *window,*table,*notebook,*frame,*label,*button,*box1,*box2;
      int i;
      GtkWidget *scrolledwindow1, *viewport1, *label2;
+     gchar *c;
 #if GTK_MAJOR_VERSION == 2
-GtkStyle *style;
+     Mainwindow *mw = MAINWINDOW(user_data);
+     PangoFontDescription *pfd;
 #endif
      ag = gtk_accel_group_new();
 
@@ -1299,7 +1374,7 @@ GtkStyle *style;
      gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
      gtk_widget_show (box2);
 
-     table = gtk_table_new (3, 6, FALSE);
+     table = gtk_table_new (4, 6, FALSE);
      gtk_box_pack_start (GTK_BOX (box2), table, TRUE, TRUE, 0);
 
      notebook = gtk_notebook_new ();
@@ -1322,13 +1397,16 @@ GtkStyle *style;
 	gtk_widget_show (viewport1);
 	gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport1);
 
-	label2 = gtk_label_new (_(help_page_contents[i]));
+	c = get_help_page_contents(i);
+	label2 = gtk_label_new (c);
+	g_free(c);
 #if GTK_MAJOR_VERSION == 2
 if (i==HELP_PAGE_SHORTCUTS)	// Keyboard tab only
 {
-	style = gtk_style_copy (gtk_widget_get_style (label2));
-	style->font_desc = pango_font_description_from_string("Monospace 11");  // Courier also works
-	gtk_widget_set_style (label2, style);
+     pfd = pango_font_description_copy_static(GTK_WIDGET(mw)->style->font_desc);
+     pango_font_description_set_family_static(pfd, "Monospace");
+     gtk_widget_modify_font(label2, pfd);
+     pango_font_description_free(pfd);
 }
 #endif
 	gtk_container_add (GTK_CONTAINER (viewport1), label2);
@@ -1397,7 +1475,7 @@ gtk_window_set_modal(GTK_WINDOW(a),TRUE);
      gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
      gtk_widget_show(c);
      c = gtk_label_new(_("Created by Magnus Hjorth (magnus.hjorth@home.se)\n"
-	  "Copyright 2002-2007, Magnus Hjorth"));
+	  "Copyright 2002-2018, Magnus Hjorth"));
      gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
      gtk_widget_show(c);
      c = gtk_hseparator_new();
@@ -1411,7 +1489,7 @@ gtk_window_set_modal(GTK_WINDOW(a),TRUE);
      c = gtk_hseparator_new();
      gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
      gtk_widget_show(c);
-     p = g_strdup_printf(_("Compiled %s %s"), __DATE__, __TIME__);
+     p = g_strdup_printf(_("Compiled %s %s"), BUILD_DATE, BUILD_TIME);
      c = gtk_label_new(p);
      g_free(p);
      gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
@@ -1470,9 +1548,29 @@ static void edit_record(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *wnd = MAINWINDOW (user_data);
      Chunk *c;
+     int noverruns, i;
+     off_t overrun_locs[10];
+     gchar s[2], *str;
      player_stop();
-     c = record_dialog_execute();
-     if (c) mainwindow_set_chunk(wnd,c,NULL);
+     c = record_dialog_execute(&noverruns, overrun_locs);
+     if (c) {
+	  wnd = mainwindow_set_chunk(wnd,c,NULL);
+	  if (noverruns > 0) {
+	       for (i=0; i<noverruns && i<10; i++) {
+		    s[0] = '0'+i;
+		    s[1] = 0;
+		    document_set_mark(wnd->doc, s,
+				      overrun_locs[i]/c->format.samplebytes);
+	       }
+	       str = g_strdup_printf
+		    ("%d overruns where detected while recording. "
+		     "This means the recording may be damaged. The "
+		     "locations where the overruns happened have been "
+		     "indicated with markers.",noverruns);
+	       user_warning(str);
+	       g_free(str);
+	  }
+     }
 }
 
 static void edit_preferences(GtkMenuItem *menuitem, gpointer user_data)
@@ -1485,6 +1583,11 @@ static void effects_combinechannels(GtkMenuItem *menuitem, gpointer user_data)
      mainwindow_show_effect_dialog(MAINWINDOW(user_data),"combine");
 }
 
+static void effects_sandwich(GtkMenuItem *menuitem, gpointer user_data)
+{
+     mainwindow_show_effect_dialog(MAINWINDOW(user_data),"sandwich");
+}
+
 static void view_timescale(GtkCheckMenuItem *checkmenuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW ( user_data );
@@ -1511,9 +1614,11 @@ static void view_vertzoom(GtkCheckMenuItem *checkmenuitem, gpointer user_data)
      if (checkmenuitem->active) {
 	  gtk_widget_show(w->vzoom_icon);
 	  gtk_widget_show(w->vzoom_slider);
+	  gtk_widget_show(GTK_WIDGET(w->vzoom_label));
      } else {
 	  gtk_widget_hide(w->vzoom_icon);
 	  gtk_widget_hide(w->vzoom_slider);
+	  gtk_widget_hide(GTK_WIDGET(w->vzoom_label));
      }
 }
 
@@ -1524,7 +1629,7 @@ static void view_speed(GtkCheckMenuItem *checkmenuitem, gpointer user_data)
      if (checkmenuitem->active) {
 	  gtk_widget_show(w->speed_icon);
 	  gtk_widget_show(w->speed_slider);
-	  gtk_widget_show(GTK_WIDGET(w->speed_label));
+	  if (w->show_labels) gtk_widget_show(GTK_WIDGET(w->speed_label));
      } else {
 	  gtk_widget_hide(w->speed_icon);
 	  gtk_widget_hide(w->speed_slider);
@@ -1532,6 +1637,29 @@ static void view_speed(GtkCheckMenuItem *checkmenuitem, gpointer user_data)
      }
 }
 
+static void view_sliderlabels(GtkCheckMenuItem *checkmenuitem, gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     w->show_labels = checkmenuitem->active;
+     if (w->vzoom_slider == NULL) return;
+     if (w->show_labels) {
+	  if (GTK_WIDGET_VISIBLE(w->vzoom_slider))
+	       gtk_widget_show(GTK_WIDGET(w->vzoom_label));
+	  if (GTK_WIDGET_VISIBLE(w->speed_slider))
+	       gtk_widget_show(GTK_WIDGET(w->speed_label));
+     } else {
+	  gtk_widget_hide(GTK_WIDGET(w->vzoom_label));
+	  gtk_widget_hide(GTK_WIDGET(w->speed_label));	  
+     }
+}
+
+static void view_bufpos(GtkCheckMenuItem *checkmenuitem, gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     w->view->show_bufpos = checkmenuitem->active;
+     gtk_widget_queue_draw(GTK_WIDGET(w->view));
+}
+
 static void edit_insertsilence(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW ( user_data );
@@ -1555,7 +1683,7 @@ static void edit_clearclipboard(GtkMenuItem *menuitem, gpointer user_data)
      gtk_object_unref(GTK_OBJECT(clipboard));
      clipboard = NULL;
      list_object_foreach(mainwindow_objects, 
-			 (GFunc)update_clipboard_items, (gpointer)0);
+			 (GFunc)update_clipboard_items, NULL);
 }
 
 static void edit_positioncursor(GtkMenuItem *menuitem, gpointer user_data)
@@ -1571,7 +1699,7 @@ static Chunk *effects_normalize_proc(Chunk *chunk, StatusBar *bar,
      gfloat *lp = (gfloat *)user_data;
      sample_t level = (lp == NULL) ? 1.0 : *lp;
      s = chunk_peak_level(chunk,bar);
-     if ((s < 0.0 && level >= 1.0) || s == 0.0) return NULL;
+     if (s < 0.0) return NULL;
      c = chunk_amplify(chunk,level/s,dither_editing,bar);
      return c;
 }
@@ -1804,7 +1932,7 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  {N_("/File/Save selection as..."),"<control>U",file_saveselection,0,NULL},
 	  { N_("/_Edit"),         NULL,         NULL,           0, "<Branch>"    },
 	  { N_("/Edit/_Undo"),    "<control>Z", edit_undo,      0, NULL          },
-	  { N_("/Edit/_Redo"),    NULL,         edit_redo,      0, NULL },
+	  { N_("/Edit/_Redo"), "<shift><control>Z", edit_redo,  0, NULL },
 	  { N_("/Edit/sep1"),     NULL,         NULL,           0, "<Separator>" },
 	  { N_("/Edit/Cu_t"),     "<control>X", edit_cut,       0, NULL          },
 	  { N_("/Edit/_Copy"),    "<control>C", edit_copy,      0, NULL          },
@@ -1834,6 +1962,8 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  { N_("/View/_Horizontal zoom"),NULL,  view_horizoom,  0, "<CheckItem>" },
 	  { N_("/View/_Vertical zoom"),NULL,    view_vertzoom,  0, "<CheckItem>" },
 	  { N_("/View/Sp_eed slider"),NULL,     view_speed,     0, "<CheckItem>" },
+	  { N_("/View/Slider labels"),NULL,     view_sliderlabels,0,"<CheckItem>"},
+	  { N_("/View/Buffer position"),NULL,   view_bufpos,    0, "<CheckItem>" },
 	  { N_("/_Cursor"),       NULL,         NULL,           0, "<Branch>"    },
 	  {N_("/Cursor/Set selection start"),"<control>Q",edit_selstartcursor,0,
 	   NULL},
@@ -1887,6 +2017,7 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  { N_("/Effects/Add channe_l"),NULL,effects_splitchannel,0,NULL    },
 	  { N_("/Effects/Ma_p channels..."),NULL,effects_mapchannels,0,NULL },
 	  {N_("/Effects/_Combine channels..."),NULL,effects_combinechannels,0,NULL},
+	  { N_("/Effects/Add channels from other file..."),NULL,effects_sandwich,0,NULL},
 	  { N_("/Effects/sep3"),  NULL,         NULL,           0, "<Separator>" },
 	  { N_("/Effects/_Speed adjustment..."),NULL,effects_speed, 0, NULL      },
 	  { N_("/Effects/Pipe through program..."),NULL,effects_pipe,0,NULL      },
@@ -1896,6 +2027,7 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  { N_("/Debug"),         NULL,         NULL,           0, "<Branch>"    },
 	  /*	  { N_("/Debug/Mark as modified"),NULL, debug_mark,     0, NULL          }, */
 	  { N_("/Debug/Dummy effect"),NULL,     debug_dummy,    0, NULL          },
+	  { N_("/Debug/Dummy effect FP"),NULL,  debug_dummy2,   0, NULL          },
 	  { N_("/Debug/Check opencount"),NULL,  debug_checkoc,  0, NULL          },
 	  { N_("/Debug/Dump chunk info"),NULL,  debug_chunkinfo,0, NULL          },
 #endif
@@ -1917,7 +2049,8 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  "/Edit/Select all", "/Cursor", "/View", "/Play/Play from cursor",
 	  "/Play/Play all", "/Play/Stop", "/Effects", 
 	  "/Effects/Normalize",
-	  "/Effects/Effects dialog..."
+	  "/Effects/Effects dialog...",
+	  "/Edit/Select none"
      };
 
      gchar *need_selection_names[] = {
@@ -2022,9 +2155,22 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  (GTK_CHECK_MENU_ITEM(item),
 	   inifile_get_gboolean(INI_SETTING_SPEED,INI_SETTING_SPEED_DEFAULT));
 							  
+     item = gtk_item_factory_get_item(item_factory,"/View/Slider labels");
+     gtk_check_menu_item_set_active
+	  (GTK_CHECK_MENU_ITEM(item),
+	   inifile_get_gboolean(INI_SETTING_SLABELS,INI_SETTING_SLABELS_DEFAULT));
+
+     item = gtk_item_factory_get_item(item_factory,"/View/Buffer position");
+     gtk_check_menu_item_set_active
+	  (GTK_CHECK_MENU_ITEM(item),
+	   inifile_get_gboolean(INI_SETTING_BUFPOS,INI_SETTING_BUFPOS_DEFAULT));
+
      item = gtk_item_factory_get_item(item_factory,"/Play/Record...");
      gtk_widget_set_sensitive(item,input_supported());
 
+     item = gtk_item_factory_get_item(item_factory,"/Edit/Delete");
+     gtk_widget_add_accelerator(item, "activate", accel_group, GDK_Delete, 0, 0);
+
      return gtk_item_factory_get_widget(item_factory,"<main>");
 }
 
@@ -2200,7 +2346,7 @@ static GtkWidget *create_toolbar(Mainwindow *w)
      b = gtk_pixmap_new(p, bmp);
      r = gtk_toolbar_append_element(
 	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_TOGGLEBUTTON,NULL,NULL,
-	  _("Follow cursor while playing"),"X",b,
+	  _("Keep view and playback together"),"X",b,
 	  GTK_SIGNAL_FUNC(followmode_toggle),w);
      if ( inifile_get_gboolean("followMode",FALSE) ) {
 	  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(r), TRUE);
@@ -2309,8 +2455,15 @@ static void mainwindow_state_changed(Document *d, gpointer user_data)
 static void mainwindow_value_changed(GtkAdjustment *adjustment, 
 				     gpointer user_data)
 {
-     document_set_view( MAINWINDOW(user_data)->doc, adjustment->value, 
-			adjustment->value + adjustment->page_size );
+     Document *d = MAINWINDOW(user_data)->doc;
+     off_t s,e;
+     s = (off_t)adjustment->value;
+     e = (off_t)(adjustment->value + adjustment->page_size);
+     /* Due to rounding, these can become out of range. 
+      * Especially for GTK1 where value and page_size are gfloats */
+     if (s < 0) s = 0;
+     if (e <= s) e = s+1; else if (e > d->chunk->length) e = d->chunk->length;
+     document_set_view(d, s, e);
 }
 
 static void mainwindow_zoom_changed(GtkAdjustment *adjustment, 
@@ -2327,7 +2480,7 @@ static void mainwindow_zoom_changed(GtkAdjustment *adjustment,
      max_samp = w->doc->chunk->length;
      target_samp = max_samp * pow(min_samp/max_samp,adjustment->value);
      follow_cursor = target_samp <= current_samp;
-     document_zoom(w->doc,current_samp/target_samp,(target_samp<=current_samp));
+     document_zoom(w->doc,current_samp/target_samp,follow_cursor);
 }
 
 static void mainwindow_vertical_zoom_changed(GtkAdjustment *adjustment,
@@ -2335,8 +2488,18 @@ static void mainwindow_vertical_zoom_changed(GtkAdjustment *adjustment,
 {
      gfloat s;
      Mainwindow *w = MAINWINDOW(user_data);
+     gchar c[32],d[32];
+     int ndec;
      s = pow(100.0, adjustment->value);
      chunk_view_set_scale(w->view, s);
+     ndec = 3-(int)(adjustment->value*2.0);
+     if (ndec > 0) {
+	  g_snprintf(d,sizeof(d),"V: %%.0%df",ndec);
+	  g_snprintf(c,sizeof(c),d,(float) s);
+     } else {
+	  g_snprintf(c,sizeof(c),"V: %d",(int)s);
+     }
+     gtk_label_set_text(w->vzoom_label,c);
 }
 
 static void mainwindow_speed_changed(GtkAdjustment *adjustment,
@@ -2344,9 +2507,14 @@ static void mainwindow_speed_changed(GtkAdjustment *adjustment,
 {
      Mainwindow *w = MAINWINDOW(user_data);
      gchar c[32];
+     gfloat f;
+     f = adjustment->value;
+#ifdef INV_SPEED     
+     f = -f;
+#endif
      if (w->doc == playing_document)
-	  player_set_speed(-adjustment->value);
-     g_snprintf(c,sizeof(c),"%d%%",(int)(-adjustment->value*100.0+0.5));
+	  player_set_speed(f);
+    g_snprintf(c,sizeof(c),"S: %d%%",(int)(f*100.0+0.5));
      gtk_label_set_text(w->speed_label,c);
 }
 
@@ -2365,11 +2533,43 @@ static void mainwindow_view_double_click(ChunkView *view,
      document_set_selection(wnd->doc,o[1],o[2]);
 }
 
+static gint hzoom_scale_press(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+     Mainwindow *mw = MAINWINDOW(user_data);
+     if (event->button == 3) {
+	  gtk_adjustment_set_value(mw->zoom_adj, 0);
+	  return TRUE;
+     }
+     return FALSE;
+}
+
+static gint vzoom_scale_press(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+     Mainwindow *mw = MAINWINDOW(user_data);
+     if (event->button == 3) {
+	  gtk_adjustment_set_value(mw->vertical_zoom_adj, 0);
+	  return TRUE;
+     }
+     return FALSE;
+}
+
+static gint speed_scale_press(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+     Mainwindow *mw = MAINWINDOW(user_data);
+     if (event->button == 3) {
+#ifdef INV_SPEED
+	  gtk_adjustment_set_value(mw->speed_adj, -1.0);
+#else
+	  gtk_adjustment_set_value(mw->speed_adj, 1.0);
+#endif
+	  return TRUE;
+     }
+     return FALSE;
+}
+
 static void mainwindow_init(Mainwindow *obj)
 {
      GtkWidget *a,*b,*c;
-     GdkPixmap *p;
-     GdkBitmap *mask;
      GtkRequisition req;
      GtkTargetEntry gte;
 
@@ -2387,6 +2587,8 @@ static void mainwindow_init(Mainwindow *obj)
      chunk_view_set_timescale(
 	  obj->view, inifile_get_gboolean(INI_SETTING_TIMESCALE,
 					  INI_SETTING_TIMESCALE_DEFAULT));
+     obj->view->show_bufpos = inifile_get_gboolean
+	  (INI_SETTING_BUFPOS,INI_SETTING_BUFPOS_DEFAULT);
      gtk_signal_connect( GTK_OBJECT(obj->view), "double-click",
 			 GTK_SIGNAL_FUNC(mainwindow_view_double_click), obj);
      obj->view_adj = GTK_ADJUSTMENT( gtk_adjustment_new ( 0,0,0,0,0,0 ));
@@ -2397,12 +2599,17 @@ static void mainwindow_init(Mainwindow *obj)
      gtk_signal_connect( GTK_OBJECT(obj->zoom_adj), "value-changed",
 			 GTK_SIGNAL_FUNC(mainwindow_zoom_changed), obj);
      obj->vertical_zoom_adj = 
-	  GTK_ADJUSTMENT(gtk_adjustment_new( 0.0,0.0,1.2,0.01,0.1,0.2) );
+       GTK_ADJUSTMENT(gtk_adjustment_new ( 0, 0, 0.2 + log (inifile_get_guint32 ("vzoomMax", 100)) / log (100.0), 0.01, 0.1, 0.2 ));
      gtk_signal_connect( GTK_OBJECT(obj->vertical_zoom_adj), "value-changed",
 			 GTK_SIGNAL_FUNC(mainwindow_vertical_zoom_changed),
 			 obj);
+#ifdef INV_SPEED
      obj->speed_adj =
-	  GTK_ADJUSTMENT(gtk_adjustment_new(-1.0,-5.0,0.19,0.01,0.1,0.2));
+	  GTK_ADJUSTMENT(gtk_adjustment_new(-1.0,-2.0,0.20,0.01,0.1,0.2));
+#else
+     obj->speed_adj =
+	  GTK_ADJUSTMENT(gtk_adjustment_new(1.0,0.0,2.2,0.01,0.1,0.2));
+#endif
      gtk_signal_connect(GTK_OBJECT(obj->speed_adj),"value-changed",
 			GTK_SIGNAL_FUNC(mainwindow_speed_changed),
 			obj);
@@ -2418,10 +2625,14 @@ static void mainwindow_init(Mainwindow *obj)
      obj->vzoom_icon = obj->vzoom_slider = NULL;
      obj->hzoom_icon = obj->hzoom_slider = NULL;
      obj->speed_icon = obj->speed_slider = NULL;
-     obj->speed_label = GTK_LABEL(gtk_label_new("500%"));     
-     gtk_misc_set_alignment(GTK_MISC(obj->speed_label),1.0,0.0);
+     obj->speed_label = GTK_LABEL(gtk_label_new("S: 500%"));
+     gtk_misc_set_alignment(GTK_MISC(obj->speed_label),0.0,0.0);
      gtk_widget_size_request(GTK_WIDGET(obj->speed_label),&req);
      gtk_widget_set_usize(GTK_WIDGET(obj->speed_label),req.width,req.height);
+     obj->vzoom_label = GTK_LABEL(gtk_label_new("V: 1.000"));
+     gtk_misc_set_alignment(GTK_MISC(obj->vzoom_label),0.0,0.0);
+     gtk_widget_size_request(GTK_WIDGET(obj->vzoom_label),&req);
+     gtk_widget_set_usize(GTK_WIDGET(obj->vzoom_label),req.width,req.height);
 
      obj->need_chunk_items = NULL;
      obj->need_selection_items = NULL;
@@ -2450,21 +2661,18 @@ static void mainwindow_init(Mainwindow *obj)
 
      gtk_table_attach(GTK_TABLE(b),c,0,1,0,1,GTK_SHRINK|GTK_FILL,GTK_FILL,0,0);
 
-     p = gdk_pixmap_colormap_create_from_xpm_d
-	  (NULL,gtk_widget_get_colormap(GTK_WIDGET(obj)),&mask,NULL,vzoom_xpm);
-     c = gtk_pixmap_new(p,mask);
+     c = bitmap_new_from_data(vzoom_bits, vzoom_width, vzoom_height);
+     bitmap_set_fg(BITMAP(c),0.8);     
      gtk_table_attach(GTK_TABLE(b),c,1,2,0,1,GTK_FILL,0,0,0);
      obj->vzoom_icon = c;
 
-     p = gdk_pixmap_colormap_create_from_xpm_d
-	  (NULL,gtk_widget_get_colormap(GTK_WIDGET(obj)),&mask,NULL,hzoom_xpm);
-     c = gtk_pixmap_new(p,mask);
+     c = bitmap_new_from_data(hzoom_bits, hzoom_width, hzoom_height);
+     bitmap_set_fg(BITMAP(c),0.8);
      gtk_table_attach(GTK_TABLE(b),c,2,3,0,1,GTK_FILL,0,0,0);
      obj->hzoom_icon = c;
 
-     p = gdk_pixmap_colormap_create_from_xpm_d
-	  (NULL,gtk_widget_get_colormap(GTK_WIDGET(obj)),&mask,NULL,speed_xpm);
-     c = gtk_pixmap_new(p,mask);
+     c = bitmap_new_from_data(speed_bits, speed_width, speed_height);
+     bitmap_set_fg(BITMAP(c),0.8);
      gtk_table_attach(GTK_TABLE(b),c,3,4,0,1,GTK_FILL,0,0,0);
      obj->speed_icon = c;
 
@@ -2473,24 +2681,44 @@ static void mainwindow_init(Mainwindow *obj)
 		      GTK_EXPAND|GTK_FILL|GTK_SHRINK,0,0);
 
      c = gtk_vscale_new ( obj->zoom_adj );
+     /* GTK1 doesn't work well with using the right mouse button press event. 
+      * As a work around, we use the release event instead */
+#if GTK_MAJOR_VERSION > 1
+     gtk_signal_connect(GTK_OBJECT(c),"button_press_event",GTK_SIGNAL_FUNC(hzoom_scale_press),obj);
+#else
+     gtk_signal_connect(GTK_OBJECT(c),"button_release_event",GTK_SIGNAL_FUNC(hzoom_scale_press),obj);
+#endif
      gtk_scale_set_digits(GTK_SCALE(c),3);
      gtk_scale_set_draw_value (GTK_SCALE(c), FALSE);
-     gtk_table_attach(GTK_TABLE(b),c,2,3,1,2,GTK_FILL,GTK_EXPAND|GTK_FILL,0,0);
+     gtk_table_attach(GTK_TABLE(b),c,2,3,1,2,0,GTK_EXPAND|GTK_FILL,0,0);
      obj->need_chunk_items = g_list_append(obj->need_chunk_items,c);
      obj->hzoom_slider = c;
 
      c = gtk_vscale_new ( obj->vertical_zoom_adj );
+#if GTK_MAJOR_VERSION > 1
+     gtk_signal_connect(GTK_OBJECT(c),"button_press_event",GTK_SIGNAL_FUNC(vzoom_scale_press),obj);
+#else
+     gtk_signal_connect(GTK_OBJECT(c),"button_release_event",GTK_SIGNAL_FUNC(vzoom_scale_press),obj);
+#endif
      gtk_scale_set_digits(GTK_SCALE(c),3);
      gtk_scale_set_draw_value (GTK_SCALE(c), FALSE);
-     gtk_table_attach(GTK_TABLE(b),c,1,2,1,2,GTK_FILL,GTK_EXPAND|GTK_FILL,0,0);
+     gtk_table_attach(GTK_TABLE(b),c,1,2,1,2,0,GTK_EXPAND|GTK_FILL,0,0);
      obj->need_chunk_items = g_list_append(obj->need_chunk_items,c);
      obj->vzoom_slider = c;
 
      c = gtk_vscale_new ( obj->speed_adj );
+#if GTK_MAJOR_VERSION > 1
+     gtk_signal_connect(GTK_OBJECT(c),"button_press_event",GTK_SIGNAL_FUNC(speed_scale_press),obj);
+#else
+     gtk_signal_connect(GTK_OBJECT(c),"button_release_event",GTK_SIGNAL_FUNC(speed_scale_press),obj);
+#endif
      gtk_scale_set_digits(GTK_SCALE(c),2);
      gtk_scale_set_draw_value (GTK_SCALE(c), FALSE);
      /* gtk_range_set_update_policy(GTK_RANGE(c),GTK_UPDATE_DELAYED); */
-     gtk_table_attach(GTK_TABLE(b),c,3,4,1,2,GTK_FILL,GTK_EXPAND|GTK_FILL,0,0);
+#ifndef INV_SPEED
+     gtk_range_set_inverted(GTK_RANGE(c),TRUE);
+#endif
+     gtk_table_attach(GTK_TABLE(b),c,3,4,1,2,0,GTK_EXPAND|GTK_FILL,0,0);
      obj->need_chunk_items = g_list_append(obj->need_chunk_items,c);
      obj->speed_slider = c;
 
@@ -2498,15 +2726,23 @@ static void mainwindow_init(Mainwindow *obj)
      gtk_table_attach(GTK_TABLE(b),c,0,1,2,3,GTK_FILL,0,0,0);
 
      c = GTK_WIDGET(obj->speed_label);
-     gtk_table_attach(GTK_TABLE(b),c,1,4,2,3,GTK_FILL,0,0,0);
+     gtk_table_attach(GTK_TABLE(b),c,1,4,3,4,GTK_FILL,0,2,0);
+
+     c = gtk_event_box_new();
+     gtk_container_add(GTK_CONTAINER(c),GTK_WIDGET (obj->statusbar));
+     gtk_table_attach(GTK_TABLE(b),c,0,1,3,4,GTK_FILL,0,0,0);
+
+     c = GTK_WIDGET(obj->vzoom_label);
+     gtk_table_attach(GTK_TABLE(b),c,1,4,2,3,GTK_FILL,0,2,0);
 
-     gtk_box_pack_start(GTK_BOX(a),GTK_WIDGET(obj->statusbar),FALSE,TRUE,0);
      gtk_adjustment_value_changed(obj->speed_adj);
+     gtk_adjustment_value_changed(obj->vertical_zoom_adj);
      gtk_widget_show_all(a);
 
      if (!inifile_get_gboolean(INI_SETTING_VZOOM,INI_SETTING_VZOOM_DEFAULT)) {
 	  gtk_widget_hide(obj->vzoom_icon);
 	  gtk_widget_hide(obj->vzoom_slider);
+	  gtk_widget_hide(GTK_WIDGET(obj->vzoom_label));
      }
 
      if (!inifile_get_gboolean(INI_SETTING_HZOOM,INI_SETTING_HZOOM_DEFAULT)) {
@@ -2520,6 +2756,12 @@ static void mainwindow_init(Mainwindow *obj)
 	  gtk_widget_hide(GTK_WIDGET(obj->speed_label));
      }
 
+     obj->show_labels = inifile_get_gboolean(INI_SETTING_SLABELS,INI_SETTING_SLABELS_DEFAULT);
+     if (!obj->show_labels) {
+	  gtk_widget_hide(GTK_WIDGET(obj->vzoom_label));
+	  gtk_widget_hide(GTK_WIDGET(obj->speed_label));
+     }
+
      gtk_window_set_policy(GTK_WINDOW(obj),FALSE,TRUE,FALSE);
      
      set_sensitive(obj->need_chunk_items, FALSE);
@@ -2538,9 +2780,9 @@ static void mainwindow_init(Mainwindow *obj)
 		       GDK_ACTION_COPY | GDK_ACTION_MOVE);
 }
 
-guint mainwindow_get_type(void)
+GtkType mainwindow_get_type(void)
 {
-static guint id=0;
+static GtkType id=0;
 if (!id) {
 	GtkTypeInfo info = {
 	     "Mainwindow",
