@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006 2008 2010, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -31,7 +31,6 @@
 #include <sys/wait.h>
 #include <fcntl.h>
 
-#define GTK_ENABLE_BROKEN /* for GtkText under GTK 2.x */
 #include <gtk/gtk.h>
 
 #include "main.h"
@@ -45,7 +44,7 @@
 struct pipe_data {
      int fds[3];
      gchar *command;
-     GtkText *error_log;
+     GtkWidget *error_log;
 };
 
 extern char **environ;
@@ -128,17 +127,24 @@ GtkType pipe_dialog_get_type(void)
      return id;
 }
 
-static void add_error_text(GtkText *t,gchar *text)
+static void add_error_text(GtkWidget *t,gchar *text)
 {
-     GtkWidget *w = GTK_WIDGET(t);
-     gtk_text_insert(t,gtk_style_get_font(w->style),w->style->fg,
-		     &(w->style->white),text,-1);     
+#if GTK_MAJOR_VERSION < 2
+     gtk_text_insert(GTK_TEXT(t),gtk_style_get_font(t->style),t->style->fg,
+		     &(t->style->white),text,-1);
+#else
+     GtkTextBuffer *tb;
+     GtkTextIter iter;
+     tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(t));
+     gtk_text_buffer_get_end_iter(tb,&iter);
+     gtk_text_buffer_insert(tb, &iter, text, -1);
+#endif
 }
 
-static GtkText *create_error_window(gchar *command)
+static GtkWidget *create_error_window(gchar *command)
 {
      GtkWidget *a,*b,*c;
-     GtkText *t;
+     GtkWidget *t;
      gchar *q;
      GtkAccelGroup* ag;
 
@@ -149,8 +155,12 @@ static GtkText *create_error_window(gchar *command)
      gtk_window_set_default_size(GTK_WINDOW(a),470,200);
      b = gtk_vbox_new(FALSE,0);
      gtk_container_add(GTK_CONTAINER(a),b);
+#if GTK_MAJOR_VERSION < 2
      c = gtk_text_new(NULL,NULL);
-     t = GTK_TEXT(c);
+#else
+     c = gtk_text_view_new();
+#endif
+     t = c;
      gtk_container_add(GTK_CONTAINER(b),c);
      c = gtk_button_new_with_label(_("Close"));
      gtk_widget_add_accelerator (c, "clicked", ag, GDK_KP_Enter, 0, 
@@ -183,6 +193,7 @@ gpointer pipe_dialog_open_pipe(gchar *command, int *fds, gboolean open_out)
      pid_t p;
      gchar *c;
      char *argv[4];
+     int i;
      
      /* Create pipes */
      if (pipe(cmd_in)==-1 || (open_out && pipe(cmd_out)==-1) || 
@@ -237,10 +248,11 @@ gpointer pipe_dialog_open_pipe(gchar *command, int *fds, gboolean open_out)
 	  close(cmd_err[1]);
 	  /* Synchronize */
 	  if (open_out) {
-	       read(0,&syncbyte,1);
-	       g_assert(syncbyte == 0xAB);
+	       i = read(0,&syncbyte,1);
+	       g_assert(i == 1 && syncbyte == 0xAB);
 	       syncbyte = 0xCD;
-	       write(1,&syncbyte,1);
+	       i = write(1,&syncbyte,1);
+	       g_assert(i == 1);
 	  }
 	  /* Execute subprocess */
 	  argv[0] = "sh";
@@ -267,9 +279,10 @@ gpointer pipe_dialog_open_pipe(gchar *command, int *fds, gboolean open_out)
      /* synchronize to child */
      if (open_out) {
 	  syncbyte = 0xAB;
-	  write(cmd_in[1],&syncbyte,1);
-	  read(cmd_out[0],&syncbyte,1);
-	  g_assert(syncbyte == 0xCD);
+	  i = write(cmd_in[1],&syncbyte,1);
+	  g_assert(i == 1);
+	  i = read(cmd_out[0],&syncbyte,1);
+	  g_assert(i == 1 && syncbyte == 0xCD);
      }
 
      pd = g_malloc(sizeof(*pd));
@@ -432,7 +445,11 @@ static Chunk *pipe_dialog_pipe_chunk_main(Chunk *chunk, gchar *command,
 	       }
 	       /* printf("Writing data...\n"); */
 	       /* Write data */
+#ifdef PIPE_BUF
 	       i = write(fds[0],outbuf+bp,MIN(bs-bp,PIPE_BUF));
+#else
+	       i = write(fds[0],outbuf+bp,MIN(bs-bp,fpathconf(fds[0], _PC_PIPE_BUF)));
+#endif
 	       /* i = write(fds[0],outbuf+bp,1); */
 	       /* printf("Finished writing.\n"); */
 	       if (i == -1) {
