@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -28,6 +28,7 @@
 #include "int_box.h"
 #include "main.h"
 #include "gettext.h"
+#include "mainloop.h"
 
 static gboolean ok_flag, destroy_flag;
 static Dataformat fmt;
@@ -57,6 +58,7 @@ Dataformat *rawdialog_execute(gchar *filename, gint filesize, guint *offset)
      GtkWidget *a,*b,*c;
      gchar *ch;
      
+     memset(&fmt,0,sizeof(fmt));
      fmt.type = DATAFORMAT_PCM;
      fmt.samplerate = 22050;
      fmt.channels = 1;
@@ -110,7 +112,7 @@ Dataformat *rawdialog_execute(gchar *filename, gint filesize, guint *offset)
      ok_flag = destroy_flag = FALSE;
      
      gtk_widget_show_all(GTK_WIDGET(w));
-     while (!destroy_flag) mainloop(TRUE);
+     while (!destroy_flag) mainloop();
      
      if (!ok_flag) return NULL;
 
