@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004 2005, Magnus Hjorth
+ * Copyright (C) 2004 2005 2006 2008 2009 2011 2012, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,18 +15,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
 /* JACK sound driver */
 
-/* Currently, this driver has some limitations: 
- *   - Output is always in the (-1.0,1.0) range.
- *   - Input is expected to be normalized so absolute values > 1.0 will be
- *     clipped.
- */
-
 #include <jack/jack.h>
 #include <jack/ringbuffer.h>
 
@@ -36,6 +30,13 @@
 /* All variables/functions prefixed with mhjack_ since the jack library
  * already uses the jack_ prefix */
 
+#define MHJACK_PLAY    0x80000000
+#define MHJACK_STOP    0x40000000
+#define MHJACK_REC     0x20000000
+#define MHJACK_SKIPREQ(x) (((x) >> 24) & 15)
+#define MHJACK_SKIPPOS(x) ((x) & 0xFFFFFF)
+#define MHJACK_NEWSKIP(x,p) (((x) & 0xF0000000) | (((x) + 0x01000000) & 0x0F000000) | (p))
+
 static struct {
      gchar *client_name;
      jack_client_t *myself;
@@ -43,12 +44,15 @@ static struct {
      guint maxoutports, maxinports;
      gchar *inportnames[8],*outportnames[8];
      jack_port_t *inports[8],*outports[8];
-     volatile gboolean is_playing, is_recording, is_stopping, is_clearing;
+     volatile guint32 procctrl;
      volatile unsigned int xrun_count;
      volatile off_t played_bytes;
+     int cb_skipcount;
      Dataformat current_format;
      size_t buffer_size;     
      jack_ringbuffer_t *buffers[8];
+     gboolean refill;
+     guint refillframes, refillpos;
 } mhjack = { 0 };
 
 struct mhjack_prefdlg {
@@ -327,19 +331,42 @@ static int mhjack_xrun_callback(void *arg)
      return 0;
 }
 
+/* Move read pointer ahead to position pos, but only if pos is within the
+ * readable part of the buffer */
+static void mhjack_jrb_skipto(jack_ringbuffer_t *rb, size_t pos)
+{
+     size_t w,r;
+     w = rb->write_ptr;
+     r = rb->read_ptr;
+     if (w >= r) {
+	  if (pos > r && pos <= w)
+	       rb->read_ptr = pos;
+     } else {
+	  if (pos > r || pos <= w)
+	       rb->read_ptr = pos;
+     }
+}
+
 static int mhjack_process_callback(jack_nframes_t nframes, void *arg)
 {     
-     guint i,first_silent_port=0;
-     gboolean xrun = FALSE, first = TRUE;
+     guint i,first_silent_port=0,skippos;
+     gboolean xrun = FALSE, first = TRUE, do_skip=FALSE;
      size_t sz = nframes * sizeof(float), sz2;
      gchar *p;
-     if (mhjack.is_clearing) {
-	  for (i=0; i<mhjack.current_format.channels; i++)
-	       if (mhjack.outports[i] != NULL)
-		    jack_ringbuffer_reset(mhjack.buffers[i]);
-     } else if (mhjack.is_playing) {
+     int pctrl;
+     pctrl = mhjack.procctrl;
+     if ((pctrl & MHJACK_PLAY) != 0) {
+	  i = MHJACK_SKIPREQ(pctrl);
+	  if (i != mhjack.cb_skipcount) {
+	       mhjack.cb_skipcount = i;
+	       do_skip = TRUE;
+	       skippos = MHJACK_SKIPPOS(pctrl);
+	  }
 	  for (i=0; i<mhjack.current_format.channels; i++) {
 	       if (mhjack.outports[i] == NULL) continue;
+	       if (do_skip) {
+		    mhjack_jrb_skipto(mhjack.buffers[i],skippos);
+	       }
 	       p = jack_port_get_buffer(mhjack.outports[i],nframes);
 	       sz2 = jack_ringbuffer_read(mhjack.buffers[i],p,sz);
 	       if (sz2 < sz) {
@@ -362,7 +389,7 @@ static int mhjack_process_callback(jack_nframes_t nframes, void *arg)
 	  memset(p,0,sz);
      }
 
-     if (mhjack.is_recording) {
+     if ((pctrl & MHJACK_REC) != 0) {
 	  for (i=0; i<mhjack.current_format.channels; i++) {
 	       if (mhjack.inports[i] == NULL) continue;
 	       p = jack_port_get_buffer(mhjack.inports[i],nframes);
@@ -371,7 +398,7 @@ static int mhjack_process_callback(jack_nframes_t nframes, void *arg)
 		    xrun = TRUE;
 	  }
      }
-     if (xrun && (mhjack.is_recording || !mhjack.is_stopping)) {
+     if (xrun && ((pctrl&MHJACK_REC) != 0 || (pctrl&MHJACK_STOP) == 0)) {
 	  console_message(_("Over/underrun in JACK driver"));
 	  mhjack.xrun_count ++;
      }
@@ -412,17 +439,42 @@ static void mhjack_autoconnect(jack_port_t **ports, int typeflag,
      g_free(c);
 }
 
+static void silent_error_callback(const char *msg)
+{
+}
+
+static void normal_error_callback(const char *msg)
+{
+     fprintf(stderr,"%s\n",msg);
+}
+
 static void mhjack_connect(gboolean silent)
 {
      gchar *c;
+     jack_status_t s;
      if (mhjack.myself == NULL) {
+
+	  if (silent)
+	       jack_set_error_function(silent_error_callback);
+
 	  /* Connect to the JACK server */
-	  mhjack.myself = jack_client_new(mhjack.client_name);
-	  if (mhjack.myself == NULL) {
+	  mhjack.myself = jack_client_open(mhjack.client_name,
+					   JackNoStartServer|JackUseExactName,
+					   &s);
+	  /* All versions of Jack don't seem to set JackNameNotUnique properly.
+	   * Testing with Jack 1.9.4 gave JackServerError|JackFailure even
+	   * though JackUseExactName was the cause of the error */
+	  if (mhjack.myself == NULL /* && (s & JackNameNotUnique) != 0 */ ) {
 	       c = g_strdup_printf("%s%d",mhjack.client_name,getpid());
-	       mhjack.myself = jack_client_new(c);
+	       if (!silent)
+		    fprintf(stderr,"Retrying with Jack client name %s\n",c);
+	       mhjack.myself = jack_client_open(c,JackNoStartServer,&s);
 	       g_free(c);
 	  }
+
+	  if (silent)
+	       jack_set_error_function(normal_error_callback);
+
 	  if (mhjack.myself == NULL) {
 	       if (!silent)
 		    user_error(_("Could not connect to the JACK server."));
@@ -485,15 +537,20 @@ static void mhjack_quit(void)
 
 /* Output */
 
-static gint mhjack_output_select_format(Dataformat *format, gboolean silent)
+static gint mhjack_output_select_format(Dataformat *format, gboolean silent,
+					GVoidFunc ready_func)
 {
-     if (format->type != DATAFORMAT_FLOAT ||
-	 format->samplesize != sizeof(float))
-	  return -1;
-
+     /* Make sure we're connected before we check the format. This is
+      * to make sure that the error message is displayed. Note that
+      * mhjack_connect call is cheap when we're already connected */
      mhjack_connect(silent);
      if (!mhjack.is_activated) return silent ? -1 : +1;
 
+     if (format->type != DATAFORMAT_FLOAT ||
+	 format->samplesize != sizeof(float) ||
+	 XOR(format->bigendian,ieee_be_compatible))
+	  return -1;
+
      if (format->samplerate != jack_get_sample_rate(mhjack.myself)) return -1;
 
      memcpy(&(mhjack.current_format),format,sizeof(Dataformat));
@@ -502,8 +559,9 @@ static gint mhjack_output_select_format(Dataformat *format, gboolean silent)
 
      mhjack_setup_buffers();     
 
-     /* We don't set the is_playing/is_recording flag here, instead we
-      * set it after the first mhjack_output_play call to avoid xruns */     
+     mhjack.refill = TRUE;
+     mhjack.refillpos = 0;
+     mhjack.refillframes = 0;
 
      return 0;
 }
@@ -517,30 +575,75 @@ static gboolean mhjack_output_suggest_format(Dataformat *format,
      memcpy(result,format,sizeof(Dataformat));
      result->type = DATAFORMAT_FLOAT;
      result->samplesize = sizeof(float);
+     result->bigendian = ieee_be_compatible;
      result->samplebytes = result->samplesize * result->channels;
      result->samplerate = jack_get_sample_rate(mhjack.myself);
+     result->packing = 0;
      return TRUE;
 }
 
+/* We have two different buffer modes during playback:
+ *   Normal mode:
+ *     Keep the buffer filled to capacity minus 1/2 second
+ *   Refill mode:
+ *     Write 1/2 second of data into the buffer and then switch into
+ *     normal mode. When switching over, playback is started if it
+ *     was stopped, and a skip request (for dropping data) can be sent.
+ *
+ * The mode only matters to the user side, the playback thread plays what's
+ * in the ring buffer.
+ *
+ * Playback skipping without gaps or underruns is implemented as follows:
+ * - When output_clear_buffers is called, we switch to refill mode but the
+ *   callback thread keeps playing from the >0.5 s of old sound currently in
+ *   the ring buffers.
+ * - When enough new data has been written into the ring buffers, a skip
+ *   request is sent to the callback thread with the new data's starting
+ *   position in the ring buffer.
+ * - The next time the player thread wakes up it sees the skip request and
+ *   jumps the read position ahead to the new data. If it has already
+ *   played past that point, it ignores the skip request.
+ */
+
+static guint mhjack_writable(void)
+{
+     int writable,i,s;
+     writable = s = mhjack_ringbuffer_space(FALSE,FALSE) / sizeof(float);
+     i = (mhjack.buffer_size / 2) / sizeof(float);
+     if (mhjack.refill) {
+	  /* Return remaining part of refill before switching to normal */
+	  if (i > mhjack.refillframes) i-=mhjack.refillframes; else i=0;
+	  if (writable > i) writable=i;
+     } else {
+	  /* Leave half the buffer for refilling later */
+	  if (writable > i) writable-=i; else writable=0;
+     }
+     /* if (writable > 0)
+	  printf("writable: s=%d refill=%d refillframes=%d -> %d\n",s,
+	  mhjack.refill,mhjack.refillframes,writable); */
+     return writable;
+}
+
 static guint mhjack_output_play(gchar *buffer, guint bufsize)
 {
      guint writable,frames,i,j;
      float fbuf[1024],fbuf2[512];
-     
+     int pctrl = mhjack.procctrl;
+
      if (bufsize == 0) {
-	  if (!mhjack.is_playing && mhjack_ringbuffer_space(FALSE,TRUE)==0) 
+	  if ((pctrl&MHJACK_PLAY) == 0 && mhjack_ringbuffer_space(FALSE,TRUE)==0)
 	       return 0;
 	  /* The caller wants to drain the buffers, therefore set the 
 	   * is_stopping flag so underrun messages aren't displayed. */
-	  mhjack.is_stopping = TRUE;
-	  mhjack.is_playing = TRUE;
+	  pctrl |= (MHJACK_PLAY | MHJACK_STOP);
+	  mhjack.procctrl = pctrl;
 	  return mhjack_ringbuffer_space(FALSE,TRUE);
      }
 
      /* write(dumpfile, buffer, bufsize); */
 
      /* Calculate room in the ring buffers */
-     writable = mhjack_ringbuffer_space(FALSE,FALSE) / sizeof(float); 
+     writable = mhjack_writable();
 
      /* Calculate how many frames we can write out */
      frames = bufsize / mhjack.current_format.samplebytes;
@@ -551,7 +654,7 @@ static guint mhjack_output_play(gchar *buffer, guint bufsize)
      
      /* Convert data to floats */
      convert_array(buffer,&(mhjack.current_format),fbuf,&dataformat_single,
-		   frames*mhjack.current_format.channels,DITHER_NONE);
+		   frames*mhjack.current_format.channels,DITHER_NONE,NULL);
 
      /* De-interleave and write data to the different ring buffers. */
 
@@ -574,11 +677,20 @@ static guint mhjack_output_play(gchar *buffer, guint bufsize)
 	  }
      }
 
-     if (!mhjack.is_playing && 
-	 mhjack_ringbuffer_space(FALSE,FALSE) < 64*sizeof(float)) {
-	  /* Let the processing function work next time it's called */
-	  mhjack.is_stopping = FALSE;
-	  mhjack.is_playing = TRUE;
+     mhjack.refillframes += frames;
+     writable -= frames;
+
+     if (writable < 64) {
+	  if (mhjack.refill) {
+	       pctrl = MHJACK_NEWSKIP(pctrl,mhjack.refillpos);
+	       mhjack.refill = FALSE;
+	  }
+	  if ((pctrl&MHJACK_PLAY)==0) {
+	       /* Let the processing function work next time it's called */
+	       pctrl |= MHJACK_PLAY;
+	       pctrl &= ~MHJACK_STOP;
+	  }
+	  mhjack.procctrl = pctrl;
      }
 
      return frames * mhjack.current_format.samplebytes;
@@ -586,32 +698,33 @@ static guint mhjack_output_play(gchar *buffer, guint bufsize)
 
 static void mhjack_clear_buffers(void)
 {
-     /* FIXME: Click-free algo for this (start filling another buffer while 
-      * clearing).. */
-     mhjack.is_clearing = TRUE;
-     while (mhjack_ringbuffer_space(FALSE,TRUE) > 0) do_yield(FALSE);
-     mhjack.is_playing = FALSE;
-     mhjack.is_clearing = FALSE;
+     mhjack.refill = TRUE;
+     mhjack.refillframes = 0;
+     mhjack.refillpos = mhjack.buffers[0]->write_ptr;
 }
 
 static gboolean mhjack_output_stop(gboolean must_flush)
 {
-     if (!mhjack.is_playing) {
+     int pctrl = mhjack.procctrl,w;
+     if ((pctrl&MHJACK_PLAY)==0) {
 	  if (mhjack_ringbuffer_space(FALSE,TRUE) == 0)
 	       return TRUE;
      }
-     mhjack.is_clearing = !must_flush;
-     mhjack.is_stopping = TRUE;
-     mhjack.is_playing = TRUE;
+     if (!must_flush) {
+	  w = mhjack.buffers[0]->write_ptr;
+	  pctrl = MHJACK_NEWSKIP(pctrl, w);
+     }
+     pctrl |= (MHJACK_STOP | MHJACK_PLAY);
+     mhjack.procctrl = pctrl;
      while (mhjack_ringbuffer_space(FALSE,TRUE) > 0) do_yield(FALSE);
-     mhjack.is_playing = FALSE;
-     mhjack.is_clearing = FALSE;
+     pctrl &= ~MHJACK_PLAY;
+     mhjack.procctrl = pctrl;
      return must_flush;
 }
 
 static gboolean mhjack_output_want_data(void)
 {
-     return (mhjack_ringbuffer_space(FALSE,FALSE) >= sizeof(float));
+     return (mhjack_writable() > 0);
 }
 
 /* Input */
@@ -659,7 +772,7 @@ static void mhjack_input_store(Ringbuf *buffer)
      /* Convert data (in case sample_t == double, otherwise, convert_array
       * just does a memcpy). Dithering never matters in this conversion. */
      convert_array(fbuf2,&dataformat_single,sbuf,&(mhjack.current_format),
-		   frames*mhjack.current_format.channels,DITHER_NONE);
+		   frames*mhjack.current_format.channels,DITHER_NONE,NULL);
      /* Add to ring buffer */
      s = ringbuf_enqueue(buffer,sbuf,frames*mhjack.current_format.samplebytes);
      g_assert(s == frames*mhjack.current_format.samplebytes);
@@ -670,14 +783,44 @@ static int mhjack_get_xrun_count(void)
      return mhjack.xrun_count;
 }
 
-static gint mhjack_input_select_format(Dataformat *format, gboolean silent)
+static GList *mhjack_input_supported_formats(gboolean *complete)
+{
+     Dataformat *fmt;
+     GList *l = NULL;
+     int i,r;
+
+     if (mhjack.myself == NULL)
+	  mhjack_connect(TRUE);
+     if (mhjack.myself == NULL) {
+	  *complete = TRUE;
+	  return NULL;
+     }
+     
+     r = jack_get_sample_rate(mhjack.myself);
+     for (i=1; i<=mhjack.maxinports; i++) {
+	  fmt = g_malloc(sizeof(*fmt));
+	  fmt->type = DATAFORMAT_FLOAT;
+	  fmt->samplesize = sizeof(float);
+	  fmt->packing = 0;
+	  fmt->channels = i;
+	  fmt->samplebytes = fmt->samplesize * fmt->channels;
+	  fmt->samplerate = r;
+	  fmt->bigendian = ieee_be_compatible;
+	  l = g_list_append(l, fmt);
+     }
+     *complete = TRUE;
+     return l;
+}
+
+static gint mhjack_input_select_format(Dataformat *format, gboolean silent,
+				       GVoidFunc ready_func)
 {
      gchar *c;
      mhjack_connect(silent);
      if (!mhjack.is_activated) return silent ? -1 : +1;
      
      if (format->samplerate != jack_get_sample_rate(mhjack.myself) ||
-	 (mhjack_output_select_format(format,TRUE))) {
+	 (mhjack_output_select_format(format,TRUE,ready_func))) {
 	  if (!silent) {
 	       c = g_strdup_printf(_("With JACK, the only supported recording "
 				   "format is "
@@ -689,12 +832,12 @@ static gint mhjack_input_select_format(Dataformat *format, gboolean silent)
 	  } else
 	       return -1;	  
      }
-     mhjack.is_recording = TRUE;
+     mhjack.procctrl |= MHJACK_REC;
      return 0;
 }
 
 /* Also used as input_stop_hint */
 static void mhjack_input_stop(void)
 {
-     mhjack.is_recording = FALSE;
+     mhjack.procctrl &= ~MHJACK_REC;
 }
