@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006 2008 2010, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -43,8 +43,10 @@
 #include "um.h"
 #include "main.h"
 #include "gettext.h"
+#include "mainloop.h"
 
-#define OSS_PCMFILE "OSSdevice"
+#define OSS_PCMFILE_PLAYBACK "OSSdevice"
+#define OSS_PCMFILE_RECORD "OSSRecDevice"
 
 #ifdef __OpenBSD__
 #define OSS_PCMFILE_DEFAULT "/dev/audio"
@@ -61,7 +63,11 @@ static int oss_format;
 static int oss_samplesize;
 static int oss_samplerate;
 static int oss_channels;
+static int oss_samplebytes_in,oss_samplebytes_out;
 static gboolean oss_noselect = FALSE;
+static gpointer play_source;
+
+static int oss_output_flush(gpointer tsource, GTimeVal *tv, gpointer user_data);
 
 static gboolean oss_init(gboolean silent)
 {
@@ -69,6 +75,7 @@ static gboolean oss_init(gboolean silent)
 	  inifile_get_guint32(INI_SETTING_SOUNDBUFSIZE,
 			      INI_SETTING_SOUNDBUFSIZE_DEFAULT));
      oss_noselect = inifile_get_gboolean(OSS_NOSELECT,OSS_NOSELECT_DEFAULT);
+     play_source = mainloop_time_source_add(NULL, oss_output_flush, NULL);
      /* When autodetecting, this is the last tested driver before the dummy,
       * so we might as well return TRUE */
      return TRUE;
@@ -76,15 +83,16 @@ static gboolean oss_init(gboolean silent)
 
 static void oss_quit(void)
 {
+     mainloop_time_source_free(play_source);
      ringbuf_free(oss_output_buffer);
 }
 
-static int oss_errdlg_open(char *filename, int flags)
+static int oss_errdlg_open(char *filename, int flags, gboolean silent)
 {
      int fd;
      gchar *c;
      fd = open(filename,flags);
-     if (fd == -1) {
+     if (fd == -1 && !silent) {
 	  c = g_strdup_printf(_("Could not open '%s': %s"),filename,strerror(errno));
 	  user_error(c);
 	  g_free(c);
@@ -92,12 +100,12 @@ static int oss_errdlg_open(char *filename, int flags)
      return fd;
 }
 
-static int oss_errdlg_ioctl(int filedes, int cmd, void *arg)
+static int oss_errdlg_ioctl(int filedes, int cmd, void *arg, gboolean silent)
 {
      int i;
      gchar *c;
      i = ioctl(filedes,cmd,arg);
-     if (i == -1 && errno != EPIPE) {
+     if (i == -1 && errno != EPIPE && !silent) {
 	  c = g_strdup_printf(_("Error in sound driver: ioctl failed: %s"),strerror(errno));
 	  user_error(c);
 	  g_free(c);
@@ -105,13 +113,13 @@ static int oss_errdlg_ioctl(int filedes, int cmd, void *arg)
      return i;
 }
 
-static gboolean oss_try_format(Dataformat *format, gboolean input)
+static gint oss_try_format(Dataformat *format, gboolean input, gboolean silent)
 {
      gchar *fname;
      int i,j;
      /* For now, we just refuse floating-point format. We should probably 
       * use a fallback format instead */
-     if (format->type == DATAFORMAT_FLOAT) return TRUE;
+     if (format->type == DATAFORMAT_FLOAT) return -1;
      /* Determine the right format */
      oss_samplesize = format->samplesize;
      switch (oss_samplesize) {
@@ -126,20 +134,21 @@ static gboolean oss_try_format(Dataformat *format, gboolean input)
 	  break;
      case 3:
      case 4:
-	  if (!format->sign) return TRUE;
+	  if (!format->sign) return -1;
+	  if (format->packing == 2) return -1;
 	  /* This is really hairy, but the AFMT_S32-constants don't seem to be 
 	   * defined in all soundcard.h files */
 	  if (format->bigendian) {
 #ifdef AFMT_S32_BE
 	       oss_format = AFMT_S32_BE;
 #else
-	       return TRUE;
+	       return -1;
 #endif
 	  } else {
 #ifdef AFMT_S32_LE
 	  oss_format = AFMT_S32_LE;
 #else
-	  return TRUE;
+	  return -1;
 #endif
 	  }
 	  break;
@@ -147,39 +156,55 @@ static gboolean oss_try_format(Dataformat *format, gboolean input)
 	  g_assert_not_reached();
      }
      /* Open the file */
-     fname = inifile_get(OSS_PCMFILE,OSS_PCMFILE_DEFAULT);
-     oss_fd = oss_errdlg_open(fname, input ? O_RDONLY : O_WRONLY);
-     if (oss_fd == -1) return TRUE;
+     fname = inifile_get(OSS_PCMFILE_PLAYBACK,OSS_PCMFILE_DEFAULT);
+     if (input)
+	  fname = inifile_get(OSS_PCMFILE_RECORD, fname);
+     oss_fd = oss_errdlg_open(fname, input ? O_RDONLY : O_WRONLY, silent);
+     if (oss_fd == -1) return silent?-1:+1;
      /* Try to set the format */
      j = oss_format;
-     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_SETFMT, &j);
-     if (i == -1 || j != oss_format) { close(oss_fd); oss_fd=-1; return TRUE; }
+     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_SETFMT, &j, silent);
+     if (i == -1 || j != oss_format) { 
+	  close(oss_fd); 
+	  oss_fd=-1; 
+	  return (i != -1 || silent)?-1:+1; 
+     }
      /* Try to set the number of channels */
      j = oss_channels = format->channels;
-     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_CHANNELS, &j);
-     if (i==-1 || j != oss_channels) { close(oss_fd); oss_fd=-1; return TRUE; }
+     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_CHANNELS, &j, silent);
+     if (i==-1 || j != oss_channels) { 
+	  close(oss_fd); 
+	  oss_fd=-1; 
+	  return (i != -1 || silent)?-1:+1; 
+     }
      /* Try to set the sample rate */
      j = oss_samplerate = format->samplerate;
-     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_SPEED, &j);
+     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_SPEED, &j, silent);
      /* FIXME: Variable tolerance (esp. for input) */
      /* Currently tolerates 5% difference between requested/received samplerate
       */
      if (i==-1 || abs(j-oss_samplerate) > oss_samplerate/20) {
-	  close(oss_fd); oss_fd=-1; return TRUE;
+	  close(oss_fd); 
+	  oss_fd=-1; 
+	  return (i != -1 || silent)?-1:+1;
      }
      /* Everything went well! */
-     return FALSE;
-}
-
-static gint oss_output_select_format(Dataformat *format, gboolean silent)
-{
-     if (oss_try_format(format,FALSE)) return -1;
+     oss_samplebytes_in = oss_samplebytes_out = oss_channels * oss_samplesize;     
+     if (oss_samplesize == 3)
+	  oss_samplebytes_out += oss_channels;
      return 0;
 }
 
-static gint oss_input_select_format(Dataformat *format, gboolean silent)
+static gint oss_output_select_format(Dataformat *format, gboolean silent,
+				     GVoidFunc ready_func)
+{
+     return oss_try_format(format,FALSE,silent);
+}
+
+static gint oss_input_select_format(Dataformat *format, gboolean silent,
+				    GVoidFunc ready_func)
 {     
-     return oss_try_format(format,TRUE) ? -1 : 0;
+     return oss_try_format(format,TRUE,silent);
 }
 
 static int oss_errdlg_write(int fd, gchar *buffer, size_t size)
@@ -208,7 +233,7 @@ static int oss_errdlg_read(int fd, gchar *buffer, size_t size)
      return s;
 }
 
-static void oss_output_flush(void)
+static int oss_output_flush(gpointer ts, GTimeVal *tv, gpointer ud)
 {
      fd_set a;
      struct timeval t = { 0 };
@@ -218,7 +243,7 @@ static void oss_output_flush(void)
      audio_buf_info info;
      /* printf("oss_output_flush: ringbuf_available: %d\n",ringbuf_available(oss_output_buffer)); */
      /* Do we have anything to write at all? */
-     if (ringbuf_available(oss_output_buffer) == 0) return;
+     if (ringbuf_available(oss_output_buffer) == 0) return 0;
      /* Do a select call and see if it's ready for writing */
      if (!oss_noselect) {
 	  FD_ZERO(&a);
@@ -227,20 +252,20 @@ static void oss_output_flush(void)
 	  if (i==-1) {
 	       c = g_strdup_printf(_("Error in sound driver: select failed: %s"),strerror(errno));
 	       user_error(c);
-	       return;
+	       return 2000;
 	  }
-	  if (i==0) return;
+	  if (i==0) return 50;
      }
 #ifdef SNDCTL_DSP_GETOSPACE
      /* Now do an ioctl call to check the number of writable bytes */
-     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_GETOSPACE, &info);
+     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_GETOSPACE, &info, TRUE);
      if (i == -1 && errno == EPIPE) {
 	  /* This is a workaround for a bug in ALSA 0.9.6 OSS emulation, where
 	   * this call can return "broken pipe", especially after a 
 	   * SNDCTL_DSP_RESET call. */	  
 	  info.fragments = 1024;
 	  info.fragsize = 1*2*3*4;
-     } else if (i == -1) return;
+     } else if (i == -1) return 0;
 #else
      /* Fill in dummy values */
      info.fragments = 1024;
@@ -251,44 +276,64 @@ static void oss_output_flush(void)
      if (u > info.fragsize)
 	  u -= u % info.fragsize;
      u = MIN(u, info.fragments*info.fragsize);
-     if (u == 0) return;
+     if (u == 0) return 50;
      /* Write it out! */
      c = g_malloc(u);
      ringbuf_dequeue(oss_output_buffer, c, u);
      i = oss_errdlg_write(oss_fd, c, u);
      g_free(c);
-     return;
+     return 10;
 }
 
 static gboolean oss_output_want_data(void)
 {
-     oss_output_flush();
-     return (ringbuf_freespace(oss_output_buffer) > 0);
+     /* oss_output_flush(); */
+     return (ringbuf_freespace(oss_output_buffer) >= oss_samplebytes_out);
 }
 
 static guint oss_output_play(gchar *buffer, guint bufsize)
 {
      guint u=0;
+     GTimeVal t;
+     guint bs0 = bufsize;
      /* 24-bit samples need padding */
      if (oss_samplesize == 3) {
-	  while (bufsize >= 3 && ringbuf_available(oss_output_buffer)>=4) {
+	  while (bufsize >= 3 && ringbuf_freespace(oss_output_buffer)>=oss_samplebytes_out) {
+	       for (u=0; u<oss_channels; u++) {
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
-	       ringbuf_enqueue(oss_output_buffer,"",1);
-	       ringbuf_enqueue(oss_output_buffer,buffer,3);
+		    ringbuf_enqueue(oss_output_buffer,"",1);
+		    ringbuf_enqueue(oss_output_buffer,buffer,3);
 #else
-	       ringbuf_enqueue(oss_output_buffer,buffer,3);
-	       ringbuf_enqueue(oss_output_buffer,"",1);
+		    ringbuf_enqueue(oss_output_buffer,buffer,3);
+		    ringbuf_enqueue(oss_output_buffer,"",1);
 #endif
-	       buffer += 3;
-	       bufsize -= 3;
-	       u += 3;
+		    buffer += 3;
+		    bufsize -= 3;
+		    u += 3;
+	       }
 	  }
      }
      /* Normal operation */
-     else if (bufsize) 
-	  u = ringbuf_enqueue(oss_output_buffer, buffer, bufsize);
-     oss_output_flush();
-     return bufsize ? u : ringbuf_available(oss_output_buffer);
+     else if (bufsize) {
+	  u = ringbuf_freespace(oss_output_buffer);
+	  if (u > bufsize) u = bufsize;
+	  /* Make sure we store whole samples */
+	  u -= u%(oss_samplebytes_out);
+	  if (u>0)
+	       u = ringbuf_enqueue(oss_output_buffer, buffer, u);	  
+	  buffer += u;
+	  bufsize -= u;
+     }
+     if (ringbuf_freespace(oss_output_buffer) < oss_samplebytes_out && 
+	 !mainloop_time_source_enabled(play_source)) {
+	  puts("Starting time source");
+	  g_get_current_time(&t);
+	  mainloop_time_source_restart(play_source,&t);
+     }
+     g_assert(bufsize == 0 || bufsize >= oss_samplebytes_in);
+     printf("oss_output_play, bs0: %d, u: %d\n",bs0,u);
+     /* oss_output_flush(); */
+     return bs0 ? u : ringbuf_available(oss_output_buffer);
 }
 
 static void oss_output_clear_buffers(void)
@@ -305,7 +350,7 @@ static gboolean oss_output_stop(gboolean must_flush)
      if (oss_fd == -1) return TRUE;
      if (must_flush)
 	  while (ringbuf_available(oss_output_buffer)>0) {
-	       oss_output_flush();
+	       oss_output_flush(NULL,NULL,NULL);
 	       do_yield(TRUE);
 	  }
      else
@@ -351,7 +396,7 @@ static void oss_input_store(Ringbuf *buffer)
 #ifdef SNDCTL_DSP_GETISPACE
      /* Now do an ioctl call to check the number of readable bytes */
      /* Note: It seems as ALSA 0.9's OSS emulation returns 0 for this call... */
-     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_GETISPACE, &info);
+     i = oss_errdlg_ioctl(oss_fd, SNDCTL_DSP_GETISPACE, &info, TRUE);
      if (i > 0 && i < u) u=i;
      else u = MIN(u,oss_samplesize*oss_channels*oss_samplerate/50);
 #else
@@ -368,13 +413,17 @@ static void oss_input_store(Ringbuf *buffer)
 
 struct oss_prefdlg { 
      GtkWindow *wnd; 
-     GtkEntry *pcmdev; 
+     GtkEntry *pcmdev_playback;
+     GtkEntry *pcmdev_record;
      GtkToggleButton *noselect;
 };
 
 static void oss_preferences_ok(GtkButton *button, struct oss_prefdlg *pd)
 {
-     inifile_set(OSS_PCMFILE,(char *)gtk_entry_get_text(pd->pcmdev));
+     inifile_set(OSS_PCMFILE_PLAYBACK,
+		 (char *)gtk_entry_get_text(pd->pcmdev_playback));
+     inifile_set(OSS_PCMFILE_RECORD,
+		 (char *)gtk_entry_get_text(pd->pcmdev_record));
      oss_noselect = gtk_toggle_button_get_active(pd->noselect);
      inifile_set_gboolean(OSS_NOSELECT,oss_noselect);
      gtk_widget_destroy(GTK_WIDGET(pd->wnd));
@@ -384,6 +433,7 @@ static void oss_preferences(void)
 {
      GtkWidget *a,*b,*c,*d;
      struct oss_prefdlg *pd;
+     gchar *q;
      pd = g_malloc(sizeof(struct oss_prefdlg));
      a = gtk_window_new(GTK_WINDOW_DIALOG);     
      gtk_window_set_modal(GTK_WINDOW(a),TRUE);
@@ -397,12 +447,21 @@ static void oss_preferences(void)
      gtk_container_add(GTK_CONTAINER(a),b);
      c = gtk_hbox_new(FALSE,3);
      gtk_container_add(GTK_CONTAINER(b),c);
-     d = gtk_label_new(_("Sound device file:"));
+     d = gtk_label_new(_("Playback device file:"));
      gtk_container_add(GTK_CONTAINER(c),d);
      d = gtk_entry_new();
-     gtk_entry_set_text(GTK_ENTRY(d),inifile_get(OSS_PCMFILE,OSS_PCMFILE_DEFAULT));
+     q = inifile_get(OSS_PCMFILE_PLAYBACK,OSS_PCMFILE_DEFAULT);
+     gtk_entry_set_text(GTK_ENTRY(d),q);
      gtk_container_add(GTK_CONTAINER(c),d);
-     pd->pcmdev = GTK_ENTRY(d);
+     pd->pcmdev_playback = GTK_ENTRY(d);
+     c = gtk_hbox_new(FALSE,3);
+     gtk_container_add(GTK_CONTAINER(b),c);
+     d = gtk_label_new(_("Recording device file:"));
+     gtk_container_add(GTK_CONTAINER(c),d);
+     d = gtk_entry_new();
+     gtk_entry_set_text(GTK_ENTRY(d),inifile_get(OSS_PCMFILE_RECORD,q));
+     gtk_container_add(GTK_CONTAINER(c),d);
+     pd->pcmdev_record = GTK_ENTRY(d);
      c = gtk_check_button_new_with_label(_("Avoid select calls (try this if "
 					 "recording locks up)"));
      gtk_container_add(GTK_CONTAINER(b),c);
