@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 2004 2005 2006, Magnus Hjorth
+ * Copyright (C) 2003 2004 2005 2006 2007 2009 2012, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with mhWaveEdit; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -47,6 +47,8 @@ static gchar *supported_effects[] = {
 };
 
 static gboolean sox_support_map[22] = { FALSE };
+static gboolean v13_mode = FALSE;
+static gboolean v14_mode = FALSE;
 
 static gchar *supported_effect_names[] = { 
      N_("Echo"),
@@ -75,16 +77,26 @@ static gchar *supported_effect_names[] = {
 };
 
 static gchar *samplesize_switch[] = { NULL,"-b","-w",NULL,"-l" };
+static gchar *samplesize_switch_v13[] = { NULL,"-1","-2","-3","-4" };
 
 #define COMPAND_LINES 6
 
 void sox_dialog_format_string(gchar *buf, guint bufsize, Dataformat *fmt)
 {
-     g_assert(fmt->type == DATAFORMAT_PCM && fmt->samplesize != 3);
+     g_assert(fmt->type == DATAFORMAT_PCM && fmt->samplesize != 3 && fmt->packing == 0);
 
-     g_snprintf(buf,bufsize,"-t raw -r %d %s %s -c %d",fmt->samplerate,
-		fmt->sign?"-s":"-u",samplesize_switch[fmt->samplesize],
-		fmt->channels);
+     if (v14_mode)
+	  g_snprintf(buf,bufsize,"-t raw -r %d -e %s -b %d -c %d",fmt->samplerate,
+		     fmt->sign?"signed":"unsigned",(8 * fmt->samplesize),
+		     fmt->channels);
+     else if (v13_mode)
+	  g_snprintf(buf,bufsize,"-t raw -r %d %s %s -c %d",fmt->samplerate,
+		     fmt->sign?"-s":"-u",samplesize_switch_v13[fmt->samplesize],
+		     fmt->channels);
+     else
+	  g_snprintf(buf,bufsize,"-t raw -r %d %s %s -c %d",fmt->samplerate,
+		     fmt->sign?"-s":"-u",samplesize_switch[fmt->samplesize],
+		     fmt->channels);
 }
 
 static Chunk *sox_dialog_apply_proc_main(Chunk *chunk, StatusBar *bar, 
@@ -92,7 +104,7 @@ static Chunk *sox_dialog_apply_proc_main(Chunk *chunk, StatusBar *bar,
 {
      SoxDialog *sd = SOX_DIALOG(user_data);
      EffectDialog *ed = &(sd->ed);
-     gchar fmt_buf[30];
+     gchar fmt_buf[45];
      gchar cmd_buf[512],*c;
      gchar *t1,*t2;
      gfloat f;
@@ -249,7 +261,7 @@ static Chunk *sox_dialog_apply_proc(Chunk *chunk, StatusBar *bar,
      Dataformat stype = { DATAFORMAT_PCM, 44100, 4, 1, 4, TRUE, IS_BIGENDIAN };
      if ((chunk->format.type == DATAFORMAT_FLOAT) || 
 	 (chunk->format.type == DATAFORMAT_PCM && 
-	  chunk->format.samplesize == 3)) {
+	  (chunk->format.samplesize == 3 || chunk->format.packing!=0))) {
 	  c = chunk_convert_sampletype(chunk,&stype);
 	  d = sox_dialog_apply_proc_main(c,bar,user_data);
 	  gtk_object_sink(GTK_OBJECT(c));
@@ -697,8 +709,9 @@ gboolean sox_dialog_register_main(gchar source_tag)
      int fd[2],fd2[2],i,j,lb_pos=0;
      pid_t p;
      gchar *c,*d,**s,**sn;
-     gchar linebuf[4096];
+     gchar linebuf[8192];
      gboolean *map;
+     int sox_maj=0;
 
      if (!program_exists("sox")) return FALSE;
 
@@ -754,6 +767,21 @@ gboolean sox_dialog_register_main(gchar source_tag)
 	  linebuf[lb_pos] = 0;
 	  if (lb_pos == 0) return TRUE;
 	  /* printf("Sox output: %s\n",linebuf); */
+	  /* Look at first line to see if it's SoX version >= 13 */
+	  c = strchr(linebuf,'\n');
+	  *c = 0;
+	  d = strstr(linebuf,"SoX Version ");
+	  if (d != NULL) sox_maj = strtol(d+12,NULL,10);
+	  d = strstr(linebuf,"SoX v");
+	  if (d != NULL && sox_maj==0) sox_maj = strtol(d+5,NULL,10);
+	  if (sox_maj > 13) {
+	       /* printf("SoX version %d detected\n",sox_maj); */
+	       v14_mode = TRUE;
+	  } else if (sox_maj > 12) {
+	       /* printf("SoX version %d detected\n",sox_maj); */
+	       v13_mode = TRUE;
+	  }
+	  *c = '\n';
 	  /* Scan for available effects */
 	  c = strstr(linebuf,"effect: ");
 	  if (c == NULL) {
@@ -761,9 +789,13 @@ gboolean sox_dialog_register_main(gchar source_tag)
 	       if (c == NULL) {
 		    c = strstr(linebuf,"SUPPORTED EFFECTS: ");
 	            if (c == NULL) {
-		          console_message(_("Unable to detect supported "
-			         	      "SoX effects"));
-		    return TRUE;
+			 c = strstr(linebuf,"\nEFFECTS: ");
+			 if (c == NULL) {
+			      console_message(_("Unable to detect supported "
+						"SoX effects"));
+			      return TRUE;
+			 }
+			 c += 10;
 	       	    } else
 		    c += 19;
 		} else
